@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2005-2009 Free Software Foundation, Inc.
+  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -18,7 +19,7 @@
 #ifndef MD5_H
 # define MD5_H
 
-# include "stdafx.h"
+# include "myserver.h"
 
 # include <md5.h>
 # include <include/base/crypt/crypt_algo.h>
