@@ -1,6 +1,6 @@
 /*
   MyServer
-  Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,12 +15,17 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
-#include <unistd.h>
-#include <include/base/unix_socket/unix_socket.h>
-
 #ifndef FORK_SERVER_H
-#define FORK_SERVER_H
+# define FORK_SERVER_H
+
+# include "myserver.h"
+
+# ifndef WIN32
+#  include <unistd.h>
+# endif
+
+# include <include/base/unix_socket/unix_socket.h>
+
 
 struct StartProcInfo;
 
@@ -33,7 +38,7 @@ class ForkServer
   const static int FLAG_STDIN_SOCKET = 8;
 
   ForkServer () {initialized = false;}
-  ~ForkServer () {}
+  ~ForkServer () {killServer ();}
 
   void killServer ();
   int startForkServer ();
@@ -47,7 +52,7 @@ class ForkServer
   int forkServerLoop (UnixSocket *socket);
 
   int executeProcess (StartProcInfo *spi, int flags,
-                      int *pid, int *port, 
+                      int *pid, int *port,
                       bool waitEnd = false);
 
   u_short getPort (){return port;}
