@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2006, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2006, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -19,7 +20,7 @@
 #ifndef UTILITY_H
 # define UTILITY_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/base/file/file.h>
 # include <include/base/string/stringutils.h>
 # include <include/base/process/process.h>
@@ -31,7 +32,7 @@ using namespace std;
 
 # define MYSERVER_KB(x) (x << 10)
 # define MYSERVER_MB(x) (x << 20)
-# define MYSERVER_SEC(x) (x * 1000)
+# define MYSERVER_SEC(x) (x * 1000000)
 
 int preparePrintError ();
 int endPrintError ();
