@@ -1,8 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2008, 2009 Free Software Foundation,
-  Inc.
+  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -20,7 +20,7 @@
 #ifndef AUTH_METHOD_H
 # define AUTH_METHOD_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/conf/security/security_manager.h>
 
 # include <include/base/crypt/crypt_algo_manager.h>
@@ -37,7 +37,7 @@ public:
 
   CryptAlgoManager *getCryptAlgoManager () {return cryptAlgoManager;}
   void setCryptAlgoManager (CryptAlgoManager *cam){cryptAlgoManager = cam;}
-  
+
 
 protected:
   bool comparePassword (const char *password, const char *savedPassword,
