@@ -1,7 +1,7 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -19,9 +19,11 @@
 #ifndef XML_VALIDATOR_H
 # define XML_VALIDATOR_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/base/hash_map/hash_map.h>
 
+# include <include/conf/security/security_cache.h>
+
 # include <include/conf/security/security_domain.h>
 # include <include/conf/security/security_manager.h>
 # include <include/server/server.h>
@@ -31,8 +33,6 @@
 
 # include <include/base/crypt/crypt_algo_manager.h>
 
-class SecurityCache;
-
 class XmlValidator : public Validator, public AuthMethod
 {
 public:
@@ -41,14 +41,14 @@ public:
 
   using Validator::getPermissionMask;
 
-  virtual int getPermissionMask (SecurityToken* st);
+  virtual int getPermissionMask (SecurityToken *st);
 
   virtual int
-  getPermissionMaskImpl (SecurityToken* st,
+  getPermissionMaskImpl (SecurityToken *st,
                          HashMap<string, SecurityDomain*> *hashedDomains,
                          AuthMethod* authMethod);
 private:
-  XmlParser* getParser (SecurityToken* st);
+  SecurityCache::CacheNode *getParser (SecurityToken *st);
   bool doCondition (xmlNodePtr node,
                     HashMap<string, SecurityDomain*> *hashedDomains);
 
@@ -60,6 +60,9 @@ private:
                  SecurityToken *st,
                  HashMap<string, SecurityDomain*> *hashedDomains);
 
+  void doSetHeader (xmlNodePtr node,
+                    SecurityToken *st);
+
   void doPermission (xmlNodePtr node,
                      SecurityToken *st,
                      HashMap<string, SecurityDomain*> *hashedDomains);
