@@ -1,8 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2007, 2009 Free Software Foundation,
-  Inc.
+  Copyright (C) 2002, 2003, 2004, 2007, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -20,7 +20,7 @@
 #ifndef FILTERS_FACTORY_H
 # define FILTERS_FACTORY_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/filter/stream.h>
 # include <include/filter/filter.h>
 # include <include/filter/filters_chain.h>
@@ -46,9 +46,9 @@ public:
   int insert (const char*, FILTERCREATE ptr);
   int insert (const char*, FiltersSource* ptr);
   Filter *getFilter (const char*);
-  FiltersChain* chain (list<string> &l, Stream* out, u_long *nbw,
+  FiltersChain* chain (list<string> &l, Stream* out, size_t *nbw,
                        int onlyNotModifiers = 0);
-  int chain (FiltersChain*, list<string> &l, Stream* out, u_long *nbw,
+  int chain (FiltersChain*, list<string> &l, Stream* out, size_t *nbw,
              int onlyNotModifiers = 0, string *accepted = NULL);
 
   FiltersFactory ();
