@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2009, 2010 Free Software Foundation,
+  Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -19,21 +20,21 @@
 #ifndef MEMORY_STREAM_H
 # define MEMORY_STREAM_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/filter/stream.h>
 # include <include/base/mem_buff/mem_buff.h>
 
 class MemoryStream : public Stream
 {
 public:
-  virtual int read (char* buffer, u_long len, u_long*);
-  virtual int write (const char* buffer, u_long len, u_long*);
-  virtual int flush (u_long*);
+  virtual int read (char *buffer, size_t len, size_t *);
+  virtual int write (const char *buffer, size_t len, size_t *);
+  virtual int flush (size_t *);
   int refresh ();
   int availableToRead ();
-  int read (Stream*, u_long len, u_long *nbw);
+  int read (Stream *, size_t len, size_t *nbw);
   MemoryStream ();
-  MemoryStream (MemBuf*);
+  MemoryStream (MemBuf *);
   virtual ~MemoryStream ();
 private:
   int internalData;
