@@ -1,8 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2007, 2009 Free Software Foundation,
-  Inc.
+  Copyright (C) 2002, 2003, 2004, 2007, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -20,7 +20,7 @@
 #ifndef HTTP_REQUEST_H
 # define HTTP_REQUEST_H
 
-# include "stdafx.h"
+# include "myserver.h"
 # include <string>
 # include <include/base/hash_map/hash_map.h>
 # include <include/protocol/http/http_header.h>
@@ -40,37 +40,10 @@ using namespace std;
 
 
 /*!
- *Structure to describe an HTTP request.
+  Structure to describe an HTTP request.
  */
 struct HttpRequestHeader : public HttpHeader
 {
-  struct Entry
-  {
-    string *name;
-    string *value;
-    Entry ()
-    {
-      name = new string ();
-      value = new string ();
-    }
-
-    Entry (string& n, string& v)
-    {
-      name = new string ();
-      value = new string ();
-
-      name->assign (n);
-      value->assign (v);
-    }
-
-    ~Entry ()
-    {
-      delete name;
-      delete value;
-
-    }
-
-  };
   string cmd;
   string ver;
   string auth;
@@ -79,8 +52,8 @@ struct HttpRequestHeader : public HttpHeader
   string uriOpts;
   char *uriOptsPtr;
   string rangeType;
-  u_long rangeByteBegin;
-  u_long rangeByteEnd;
+  off_t rangeByteBegin;
+  off_t rangeByteEnd;
   int uriEndsWithSlash;
 
   /*! Digest authorization scheme stuff.  */
@@ -95,8 +68,6 @@ struct HttpRequestHeader : public HttpHeader
   char digestQop[16+1];
   char digestNc[10+1];
 
-
-  HashMap<string, HttpRequestHeader::Entry*> other;
   virtual string* getValue (const char* name, string* out);
   virtual string* setValue (const char* name, const char* in);
 
