@@ -1,7 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -19,7 +20,7 @@
 #ifndef HTTP_RESPONSE_H
 # define HTTP_RESPONSE_H
 
-# include "stdafx.h"
+# include "myserver.h"
 
 # include <string>
 # include <include/base/hash_map/hash_map.h>
@@ -33,7 +34,7 @@ using namespace std;
 
 
 /*!
- * Structure to describe an HTTP response
+  Structure to describe an HTTP response
  */
 struct HttpResponseHeader : public HttpHeader
 {
@@ -43,37 +44,11 @@ struct HttpResponseHeader : public HttpHeader
   const static int CLIENT_ERROR = 400;
   const static int SERVER_ERROR = 500;
 
-  struct Entry
-  {
-    string *name;
-    string *value;
-    Entry ()
-    {
-      name = new string ();
-      value = new string ();
-    }
-
-    Entry (string& n, string& v)
-    {
-      name = new string ();
-      value = new string ();
-
-      name->assign (n);
-      value->assign (v);
-    }
-    ~Entry ()
-    {
-      delete name;
-      delete value;
-
-    }
-  };
   int httpStatus;
   string ver;
   string contentLength;
   string errorType;
 
-  HashMap<string,HttpResponseHeader::Entry*> other;
   HttpResponseHeader ();
   ~HttpResponseHeader ();
 
