@@ -1,8 +1,8 @@
 /* -*- mode: c++ -*- */
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2008, 2009 Free Software Foundation,
-  Inc.
+  Copyright (C) 2002, 2003, 2004, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -19,7 +19,7 @@
 
 #ifndef HTTPS_H
 # define HTTPS_H
-# include "stdafx.h"
+# include "myserver.h"
 # include <include/protocol/http/http.h>
 
 class Https : public Http
@@ -33,7 +33,7 @@ public:
 };
 
 /*!
- *Adapter class to make Https reentrant.
+  Adapter class to make Https reentrant.
  */
 class HttpsProtocol : public Protocol
 {
