@@ -1,34 +1,32 @@
 /*
-MyServer
-Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
+  MyServer
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free
+  Software Foundation, Inc.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 
-#include "stdafx.h"
+#include "myserver.h"
 #include <include/base/file/files_utility.h>
+#include <include/base/read_directory/rec_read_directory.h>
 #include <include/base/utility.h>
 #include <include/base/string/stringutils.h>
 
-extern "C"
-{
 #include <sys/stat.h>
-}
+
 
 #ifndef WIN32
-extern "C"
-{
 # include <fcntl.h>
 # include <unistd.h>
 # include <sys/types.h>
@@ -39,19 +37,21 @@ extern "C"
 # include <string.h>
 # include <math.h>
 # include <time.h>
-}
+
 #endif
 
 #include <string>
 #include <sstream>
 #include <memory>
 
+#include <include/base/exceptions/checked.h>
+
 using namespace std;
 
 string FilesUtility::tmpPath;
 
 /*!
- * Constructor body.
+  Constructor body.
  */
 FilesUtility::FilesUtility ()
 {
@@ -59,19 +59,19 @@ FilesUtility::FilesUtility ()
 }
 
 /*!
- * This funtion iterates through every character of the path
- * The first IF tries to clear out the bars, if it finds one,
- * just advances one character and starts the cycle again
- * The second IF tries to find at least two dots.
- * if it finds only 1, does nothing,
- * WIN32 if it finds 2 or more rec=rec-(number of dots -1)
- * *NIX if it finds 2, decrements rec, if it finds more,
- * increments it considering it's a name if it ends with
- * something else other then a bar of a NULL,
- * then it's a path of the form "/...qwerty/" that should be considered rec++
- * The last ELSE, catches the rest and advances to the next bar.
- * Return the recursion of the path.
- * \param path The file name.
+  This function iterates through every character of the path
+  The first IF tries to clear out the bars, if it finds one,
+  just advances one character and starts the cycle again
+  The second IF tries to find at least two dots.
+  if it finds only 1, does nothing,
+  WIN32 if it finds 2 or more rec=rec-(number of dots -1)
+  *NIX if it finds 2, decrements rec, if it finds more,
+  increments it considering it's a name if it ends with
+  something else other then a bar of a NULL,
+  then it's a path of the form "/...qwerty/" that should be considered rec++
+  The last ELSE, catches the rest and advances to the next bar.
+  Return the recursion of the path.
+  \param path The file name.
  */
 int FilesUtility::getPathRecursionLevel (const char* path)
 {
@@ -135,40 +135,31 @@ int FilesUtility::getPathRecursionLevel (const char* path)
 
 
 /*!
- * Rename the file [BEFORE] to [AFTER]. Returns 0 on success.
- * \param before The old file name.
- * \param after The new file name.
+  Rename the file [BEFORE] to [AFTER]. Returns 0 on success.
+  \param before The old file name.
+  \param after The new file name.
  */
 int FilesUtility::renameFile (const char* before, const char* after)
 {
-#ifdef WIN32
-  return MoveFile (before, after) ? 0 : 1;
-#else
-  return rename (before, after);
-#endif
+  return checked::rename (before, after);
 }
 
 /*!
- * Copy the file from [SRC] to [DEST]. Returns 0 on success.
- * \param src The source file name.
- * \param dest The destination file name.
- * \param overwrite Overwrite the dest file if already exists?
+  Copy the file from [SRC] to [DEST]. Returns 0 on success.
+  \param src The source file name.
+  \param dest The destination file name.
+  \param overwrite Overwrite the dest file if already exists?
  */
 int FilesUtility::copyFile (const char* src, const char* dest, int overwrite)
 {
-#ifdef WIN32
-  return CopyFile (src, dest, overwrite ? FALSE : TRUE) ? 0 : 1;
-#else
   File srcFile;
   File destFile;
-  char buffer[4096];
-  size_t dim;
 
   if (srcFile.openFile (src, File::READ))
     return -1;
 
   if (destFile.openFile (dest, File::WRITE
-                         | (overwrite ? File::FILE_CREATE_ALWAYS : 0)))
+                         | (overwrite ? File::FILE_OPEN_ALWAYS : 0)))
     {
       srcFile.close ();
       return -1;
@@ -180,88 +171,163 @@ int FilesUtility::copyFile (const char* src, const char* dest, int overwrite)
   destFile.close ();
 
   return ret;
-#endif
 }
 
 /*!
- * Copy the file from [SRC] to [DEST]. Returns 0 on success.
- * \param src The source File.
- * \param dest The destination File.
+  Copy the directory from [SRC] to [DEST]. Returns 0 on success.
+  \param src The source directory name.
+  \param dest The destination directory name.
+  \param overwrite Overwrite the dest directory if already exists?
  */
-int FilesUtility::copyFile (File& src, File& dest)
+int FilesUtility::copyDir (string const & src, string const & dest, int overwrite)
 {
-  char buffer[4096];
-  u_long nbr, nbw;
-  int ret;
+  string finaldest, srcpath;
+  File srcFile, destFile;
 
-#ifdef HAVE_POSIX_FALLOCATE
-  if (posix_fallocate (dest.getHandle (), dest.getSeek (),
-                       src.getFileSize () - src.getSeek ()))
-    return -1;
-#endif
+  int loc = src.rfind ("/");
+  loc++;
 
-  for (;;)
-  {
-    ret = src.read (buffer, 4096, &nbr);
-    if (ret)
-      return -1;
+  RecReadDirectory recTree;
+  recTree.clearTree ();
+  recTree.fileTreeGenerate (src.c_str ());
 
-    if (!nbr)
-      break;
+  while (recTree.nextMember ())
+    {
+      if (recTree.getInfo () == 6)
+        continue;
+
+      srcpath = string (recTree.getPath ());
+      finaldest = dest + "/" + srcpath.substr (loc);
+
+      /* Check if already exists.  */
+      if (FilesUtility::nodeExists (finaldest.c_str ()))
+        return 412;
+
+      if (recTree.getInfo () == 8)
+        {
+          srcFile.openFile (srcpath.c_str (), File::READ);
+          destFile.openFile (finaldest.c_str (), File::WRITE | File::FILE_OPEN_ALWAYS);
+          copyFile (srcFile, destFile);
+        }
+
+      else if (recTree.getInfo () == 1)
+        mkdir (finaldest.c_str ());
+    }
 
-    ret = dest.writeToFile (buffer, nbr, &nbw);
-    if (ret)
-      return -1;
-  }
   return 0;
 }
 
 /*!
- * Delete an existing file passing the path.
- * Return a non-null value on errors.
- * \param filename The file to delete.
+  Copy the file from [SRC] to [DEST]. Returns 0 on success.
+  \param src The source File.
+  \param dest The destination File.
+*/
+int FilesUtility::copyFile (File& src, File& dest)
+{
+  char *buffer = new char[4096];
+  size_t nbr, nbw;
+
+  try
+    {
+#ifdef HAVE_POSIX_FALLOCATE
+      if (posix_fallocate (dest.getHandle (), dest.getSeek (),
+                           src.getFileSize () - src.getSeek ()))
+        {
+          delete [] buffer;
+          return -1;
+        }
+#endif
+
+      for (;;)
+        {
+          src.read (buffer, 4096, &nbr);
+
+          if (! nbr)
+            break;
+
+          dest.writeToFile (buffer, nbr, &nbw);
+        }
+
+      delete [] buffer;
+    }
+  catch (...)
+    {
+      delete [] buffer;
+      throw;
+    }
+
+  return 0;
+}
+
+/*!
+  Delete an existing file passing the path.
+  \param filename The file to delete.
  */
 int FilesUtility::deleteFile (const char *filename)
 {
-  int ret;
-#ifdef WIN32
-  ret = DeleteFile (filename);
-  if (ret)
-    return 0;
-#else
-  ret = remove (filename);
-
-  if (ret && errno == ENOENT)
-    ret = 0;
-#endif
-  return ret;
+  checked::remove (filename);
+  return 0;
 }
 
 /*!
- * Returns a non-null value if the path is a directory.
- * \param filename The path to check.
+  Delete the directory [SRC]. Returns 0 on success.
+  \param src The source directory name.
+ */
+int FilesUtility::deleteDir (string const & src)
+{
+  string srcpath;
+  RecReadDirectory recTree;
+  recTree.clearTree ();
+  recTree.fileTreeGenerate (src.c_str ());
+
+  while (recTree.nextMember ())
+    {
+      if (recTree.getInfo () == 1)
+        continue;
+
+      FilesUtility::deleteFile (recTree.getPath ());
+    }
+
+  return 0;
+}
+
+/*!
+  Return the result of `nodeExists (PATH) && !isDirectory (PATH)' using a
+  single stat.
+
+  \param path Parameter to `stat'.
+ */
+bool FilesUtility::notDirectory (const char *path)
+{
+  struct stat F_Stats;
+  int err = ::stat (path, &F_Stats);
+  if (err < 0)
+    {
+      if (errno == ENOENT || errno == ENOTDIR)
+        return true;
+
+      /* Raise an exception.  */
+      checked::checkError (err);
+    }
+
+  return (S_ISDIR (F_Stats.st_mode)) ? 0 : 1;
+}
+
+
+/*!
+  Returns a non-null value if the path is a directory.
+  \param filename The path to check.
  */
 int FilesUtility::isDirectory (const char *filename)
 {
-#ifdef WIN32
-  u_long fa = GetFileAttributes (filename);
-  if (fa != INVALID_FILE_ATTRIBUTES)
-    return (fa & FILE_ATTRIBUTE_DIRECTORY)?1:0;
-  else
-    return 0;
-#else
   struct stat F_Stats;
-  int ret = stat (filename, &F_Stats);
-  if (ret < 0)
-    return 0;
-
-  return (S_ISDIR (F_Stats.st_mode))? 1 : 0;
-#endif
+  checked::stat (filename, &F_Stats);
+  return (S_ISDIR (F_Stats.st_mode)) ? 1 : 0;
 }
 
 /*!
- * Returns a non-null value if the given path is a link.
- * \param filename The path to check.
+  Returns a non-null value if the given path is a link.
+  \param filename The path to check.
  */
 int FilesUtility::isLink (const char* filename)
 {
@@ -269,54 +335,35 @@ int FilesUtility::isLink (const char* filename)
   return 0;
 #else
   struct stat F_Stats;
-  int ret = lstat (filename, &F_Stats);
-  if (ret < 0)
-    return 0;
-
-  return (S_ISLNK (F_Stats.st_mode))? 1 : 0;
+  checked::lstat (filename, &F_Stats);
+  return S_ISLNK (F_Stats.st_mode) ? 1 : 0;
 #endif
 
 }
 
 /*!
- * Returns a non-null value if the given path is a valid file.
- * \param filename The path to check.
+  Returns a non-null value if the given path is a valid file.
+  \param filename The path to check.
  */
-int FilesUtility::fileExists (const char* filename)
+int FilesUtility::nodeExists (const char* filename)
 {
-#ifdef WIN32
-  HANDLE hFile = CreateFile (filename, GENERIC_READ, FILE_SHARE_READ,
-                             NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL
-                             | FILE_FLAG_BACKUP_SEMANTICS, NULL);
-
-  int nRet = hFile != INVALID_HANDLE_VALUE ? 1 : 0;
-  CloseHandle (hFile);
-  return nRet;
-#else
   struct stat fstats;
-  int ret = stat (filename, &fstats);
-  if (ret < 0)
+  if (::stat (filename, &fstats) < 0)
     return 0;
-  /*! Return 1 if it is a regular file or a directory.  */
-  return (S_ISREG (fstats.st_mode) | S_ISDIR (fstats.st_mode))? 1 : 0;
-#endif
+
+  return 1;
 }
 
 /*!
- * Returns the time of the last modify to the file.
- * Returns -1 on errors.
- * \param filename The path to check.
+  Returns the time of the last modify to the file.
+  Returns -1 on errors.
+  \param filename The path to check.
  */
 time_t FilesUtility::getLastModTime (const char *filename)
 {
   int res;
-#ifdef WIN32
-  struct _stat sf;
-  res = _stat (filename, &sf);
-#else
   struct stat sf;
   res = stat (filename,&sf);
-#endif
 
   if (res == 0)
     return sf.st_mtime;
@@ -325,20 +372,16 @@ time_t FilesUtility::getLastModTime (const char *filename)
 }
 
 /*!
- * Returns the time of the file creation.
- * Returns -1 on errors.
- * \param filename The path to check.
+  Returns the time of the file creation.
+  Returns -1 on errors.
+  \param filename The path to check.
  */
 time_t FilesUtility::getCreationTime (const char *filename)
 {
   int res;
-#ifdef WIN32
-  struct _stat sf;
-  res = _stat (filename, &sf);
-#else
   struct stat sf;
   res = stat (filename, &sf);
-#endif
+
   if (res == 0)
     return sf.st_ctime;
   else
@@ -346,20 +389,15 @@ time_t FilesUtility::getCreationTime (const char *filename)
 }
 
 /*!
- * Returns the time of the last access to the file.
- * Returns -1 on errors.
- * \param filename The path to check.
+  Returns the time of the last access to the file.
+  Returns -1 on errors.
+  \param filename The path to check.
  */
 time_t FilesUtility::getLastAccTime (const char *filename)
 {
   int res;
-#ifdef WIN32
-  struct _stat sf;
-  res = _stat (filename, &sf);
-#else
   struct stat sf;
   res = stat (filename, &sf);
-#endif
   if (res == 0)
     return sf.st_atime;
   else
@@ -367,69 +405,25 @@ time_t FilesUtility::getLastAccTime (const char *filename)
 }
 
 /*!
- * Change the owner of the current file, use the value -1 for uid or gid
- * to do not change the value. Return 0 on success.
- * \param filename The path to the file to chown.
- * \param uid The user id.
- * \param gid the group id.
+  Change the owner of the current file, use the value -1 for uid or gid
+  to do not change the value. Return 0 on success.
+  \param filename The path to the file to chown.
+  \param uid The user id.
+  \param gid the group id.
  */
 int FilesUtility::chown (const char* filename, string &uid, string &gid)
 {
 #ifndef WIN32
-  return ::chown (filename, Process::getUid (uid.c_str ()),
-                  Process::getGid (gid.c_str ())) ? 1 : 0;
+  return checked::chown (filename, Process::getUid (uid.c_str ()),
+                        Process::getGid (gid.c_str ())) ? 1 : 0;
 #endif
   return 0;
 }
 
 /*!
- *Get the length of the file in the path.
- *\param path The full path where get the filename length.
- *\param filename A pointer to the start of the file name.
- */
-int FilesUtility::getFilenameLength (const char *path, int *filename)
-{
-  int splitpoint, i = 0, j = 0;
-  splitpoint = static_cast<int>(strlen (path) - 1);
-  while ((splitpoint > 0) && (path[splitpoint] != '/'))
-    splitpoint--;
-  *filename = splitpoint + 1;
-  return *filename;
-}
-
-/*!
- * Get the filename from a path.
- * Be sure that the filename buffer is at least getFilenameLength (...) bytes
- * before call this function.
- * \param path The full path to the file.
- * \param filename A buffer to fullfill with the file name.
- */
-void FilesUtility::getFilename (const char *path, char *filename)
-{
-  int splitpoint, i = 0, j = 0;
-  splitpoint = static_cast<int>(strlen (path) - 1);
-  while ((splitpoint > 0) && (path[splitpoint] != '/'))
-    splitpoint--;
-  if ((splitpoint == 0) && (path[splitpoint] != '/'))
-    strcpy (filename, path);
-  else
-    {
-      splitpoint++;
-      i=splitpoint;
-      while (path[i] != 0)
-        {
-          filename[j] = path[i];
-          j++;
-          i++;
-        }
-      filename[j] = 0;
-    }
-}
-
-/*!
- * Get the filename from a path.
- * \param path The full path to the file.
- * \param filename A buffer to fullfill with the file name.
+  Get the filename from a path.
+  \param path The full path to the file.
+  \param filename A buffer to fullfill with the file name.
  */
 void FilesUtility::getFilename (string const &path, string& filename)
 {
@@ -441,11 +435,11 @@ void FilesUtility::getFilename (string const &path, string& filename)
 }
 
 /*!
- * Use this function before call splitPath to be sure that the buffers
- * dir and filename are bigger enough to contain the data.
- * \param path The full path to the file.
- * \param dir The directory part length of the path.
- * \param filename The length of the buffer needed to contain the file name.
+  Use this function before call splitPath to be sure that the buffers
+  dir and filename are bigger enough to contain the data.
+  \param path The full path to the file.
+  \param dir The directory part length of the path.
+  \param filename The length of the buffer needed to contain the file name.
  */
 void FilesUtility::splitPathLength (const char *path, int *dir, int *filename)
 {
@@ -464,11 +458,11 @@ void FilesUtility::splitPathLength (const char *path, int *dir, int *filename)
 }
 
 /*!
- * Splits a file path into a directory and filename.
- * Path is an input value while dir and filename are the output values.
- * \param path The full path to the file.
- * \param dir The directory part of the path.
- * \param filename A buffer to fullfill with the file name.
+  Splits a file path into a directory and filename.
+  Path is an input value while dir and filename are the output values.
+  \param path The full path to the file.
+  \param dir The directory part of the path.
+  \param filename A buffer to fullfill with the file name.
  */
 void FilesUtility::splitPath (const char *path, char *dir, char *filename)
 {
@@ -500,10 +494,10 @@ void FilesUtility::splitPath (const char *path, char *dir, char *filename)
 }
 
 /*!
- * Split a path in a dir and a filename.
- * \param path The full path to the file.
- * \param dir The directory part of the path.
- * \param filename A buffer to fullfill with the file name.
+  Split a path in a dir and a filename.
+  \param path The full path to the file.
+  \param dir The directory part of the path.
+  \param filename A buffer to fullfill with the file name.
  */
 void FilesUtility::splitPath (string const &path, string& dir, string& filename)
 {
@@ -530,10 +524,10 @@ void FilesUtility::splitPath (string const &path, string& dir, string& filename)
 }
 
 /*!
- * Get the file extension passing its path.
- * Save in ext all the bytes afer the last dot (.) in filename.
- * \param ext The buffer to fullfill with the file extension.
- * \param filename The path to the file.
+  Get the file extension passing its path.
+  Save in ext all the bytes afer the last dot (.) in filename.
+  \param ext The buffer to fullfill with the file extension.
+  \param filename The path to the file.
  */
 void FilesUtility::getFileExt (char* ext, const char* filename)
 {
@@ -550,10 +544,10 @@ void FilesUtility::getFileExt (char* ext, const char* filename)
 }
 
 /*!
- * Get the file extension passing its path.
- * Save in ext all the bytes afer the last dot (.) in filename.
- * \param ext The buffer to fullfill with the file extension.
- * \param filename The path to the file.
+  Get the file extension passing its path.
+  Save in ext all the bytes afer the last dot (.) in filename.
+  \param ext The buffer to fullfill with the file extension.
+  \param filename The path to the file.
  */
 void FilesUtility::getFileExt (string& ext, string const &filename)
 {
@@ -565,11 +559,11 @@ void FilesUtility::getFileExt (string& ext, string const &filename)
 }
 
 /*!
- * Get the file path in the short form of the specified file.
- * Return -1 on errors.
- * \param filePath The path to use.
- * \param out The buffer where write.
- * \param buffersize The buffer length.
+  Get the file path in the short form of the specified file.
+  Return -1 on errors.
+  \param filePath The path to use.
+  \param out The buffer where write.
+  \param buffersize The buffer length.
  */
 int FilesUtility::getShortFileName (char *filePath, char *out, int buffersize)
 {
@@ -585,11 +579,11 @@ int FilesUtility::getShortFileName (char *filePath, char *out, int buffersize)
 }
 
 /*!
- * Complete the path of the file.
- * Return non-zero on errors.
- * \param fileName The buffer to use.
- * \param size The new buffer size.
- * \param dontRealloc Don't realloc a new buffer.
+  Complete the path of the file.
+  Return non-zero on errors.
+  \param fileName The buffer to use.
+  \param size The new buffer size.
+  \param dontRealloc Don't checked::realloc a new buffer.
  */
 int FilesUtility::completePath (char **fileName, int *size, int dontRealloc)
 {
@@ -664,9 +658,9 @@ int FilesUtility::completePath (char **fileName, int *size, int dontRealloc)
 }
 
 /*!
- * Complete the path of the file.
- * Return non-zero on errors.
- * \param fileName The file name to complete.
+  Complete the path of the file.
+  Return non-zero on errors.
+  \param fileName The file name to complete.
  */
 int FilesUtility::completePath (string &fileName)
 {
@@ -701,35 +695,35 @@ int FilesUtility::completePath (string &fileName)
 }
 
 /*!
- * Create a new directory.
- * Return non-zero on errors.
- * \param path The path to the directory to create.
+  Create a new directory.
+  Return non-zero on errors.
+  \param path The path to the directory to create.
  */
 int FilesUtility::mkdir (const char *path)
 {
 #ifdef WIN32
   return CreateDirectory (path, NULL) ? 0 : -1;
 #else
-  return ::mkdir (path, S_IRUSR | S_IWUSR);
+  return checked::mkdir (path, 0777);
 #endif
 }
 
 /*!
- * Delete a directory.
- * Return non-zero on errors.
- * \param path The directory to remove.
+  Delete a directory.
+  Return non-zero on errors.
+  \param path The directory to remove.
  */
 int FilesUtility::rmdir (const char *path)
 {
 #ifdef WIN32
   return RemoveDirectory (path) ? 0 : -1;
 #else
-  return ::rmdir (path);
+  return checked::rmdir (path);
 #endif
 }
 
 /*!
- * Set the temporary path to a default one.
+  Set the temporary path to a default one.
  */
 void FilesUtility::resetTmpPath ()
 {
@@ -759,20 +753,15 @@ void FilesUtility::resetTmpPath ()
 
 
 /*!
- *Create an unique temporary file name.  This function doesn't create
- *or open the file but generates only its name.
- *\param tid Caller thread id.
- *\param out Output string.
+  Create an unique temporary file name mask that can be used by mkstemp.
+  \param tid Caller thread id.
+  \param out Output string.
  */
 void FilesUtility::temporaryFileName (u_long tid, string &out)
 {
-  ostringstream stream;
-  static u_long counter = 1;
-  counter++;
-
   if (tmpPath.length () == 0)
     tmpPath.assign (getdefaultwd (0, 0));
 
-  stream << tmpPath << "/myserver_" << counter  << "_" << tid << ".tmp";
-  out.assign (stream.str ());
+  out.assign (tmpPath);
+  out.append ("/myserver_XXXXXX");
 }
