@@ -1,6 +1,6 @@
 /*
   MyServer
-  Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,30 +15,30 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 
 #include <include/base/process/fork_server.h>
 #include <include/base/file/files_utility.h>
 #include <include/base/utility.h>
 
+#include <include/base/exceptions/checked.h>
+
 #ifndef WIN32
-extern "C"
-{
 # include <unistd.h>
 # include <sys/wait.h>
-}
+
 #endif
 /*!
- *Write a string to the socket.
- *The string length is sent before the content.
- *
- *\param socket Socket where write.
- *\param str string to write.
- *\param len string length.
+  Write a string to the socket.
+  The string length is sent before the content.
+
+  \param socket Socket where write.
+  \param str string to write.
+  \param len string length.
  */
 int ForkServer::writeString (Socket *socket, const char* str, int len)
 {
-  u_long nbw;
+  size_t nbw;
 
   if (str == NULL)
     len = 0;
@@ -53,13 +53,13 @@ int ForkServer::writeString (Socket *socket, const char* str, int len)
 }
 
 /*!
- *Write an integer on the specified socket.
- *\param socket Socket to use.
- *\param num Integer to write.
+  Write an integer on the specified socket.
+  \param socket Socket to use.
+  \param num Integer to write.
  */
 int ForkServer::writeInt (Socket *socket, int num)
 {
-  u_long nbw;
+  size_t nbw;
 
   if (socket->write ((const char*)&num, 4, &nbw))
     return 1;
@@ -68,15 +68,15 @@ int ForkServer::writeInt (Socket *socket, int num)
 }
 
 /*!
- *Read an integer from the socket.
- *
- *\param sock Socket where read.
- *\param dest integer where write
- *\return 0 on success.
+  Read an integer from the socket.
+
+  \param sock Socket where read.
+  \param dest integer where write
+  \return 0 on success.
  */
 int ForkServer::readInt (Socket *sock, int *dest)
 {
-  u_long nbr;
+  size_t nbr;
 
   if (sock->read ((char*)dest, 4, &nbr) || nbr < 4)
     {
@@ -87,20 +87,20 @@ int ForkServer::readInt (Socket *sock, int *dest)
 }
 
 /*!
- *Read a string from the socket.
- *The destination buffer is allocated here.
- *It must be freed by the caller.
- *
- *\param sock socket to use for read.
- *\param out destination buffer pointer.
- *\return 0 on success.
+  Read a string from the socket.
+  The destination buffer is allocated here.
+  It must be freed by the caller.
+
+  \param sock socket to use for read.
+  \param out destination buffer pointer.
+  \return 0 on success.
  */
 int ForkServer::readString (Socket *sock, char **out)
 {
   int len;
-  u_long nbr;
+  size_t nbr;
 
-  if (sock->read ((char*)&len, 4, &nbr) || nbr < 4)
+  if (sock->read ((char*) &len, 4, &nbr) || nbr < 4)
     {
       return -1;
     }
@@ -108,7 +108,7 @@ int ForkServer::readString (Socket *sock, char **out)
   *out = new char[len + 1];
   (*out)[len] = '\0';
 
-  if (len && (sock->read (*out, len, &nbr) || nbr < len))
+  if (len && (sock->read (*out, len, &nbr) || (int) nbr < len))
     {
       delete [] *out;
       return -1;
@@ -118,12 +118,12 @@ int ForkServer::readString (Socket *sock, char **out)
 }
 
 /*!
- *Handle a request on the socket.
+  Handle a request on the socket.
  */
 int ForkServer::handleRequest (Socket *sock)
 {
 #ifndef WIN32
-  int ret, flags, stdIn = -1, stdOut = -1, stdErr = -1;
+  int flags, stdIn = -1, stdOut = -1, stdErr = -1;
   char *gid, *uid;
   int stdInPort = 0;
   char *exec;
@@ -207,24 +207,24 @@ int ForkServer::handleRequest (Socket *sock)
   delete [] chroot;
 
   if (flags & FLAG_USE_IN)
-    close (stdIn);
+    checked::close (stdIn);
 
   if (flags & FLAG_USE_OUT)
-    close (stdOut);
+    checked::close (stdOut);
 
   if (flags & FLAG_USE_ERR)
-    close (stdErr);
+    checked::close (stdErr);
 #endif
 
   return 0;
 }
 
 /*!
- *Entry point for the fork server.
- *Listen for new connections on the specified socket.
- *
- *\param serverSocket Socket where wait for new connections.
- *\return 0 on success.
+  Entry point for the fork server.
+  Listen for new connections on the specified socket.
+
+  \param serverSocket Socket where wait for new connections.
+  \return 0 on success.
  */
 int ForkServer::forkServerLoop (UnixSocket *serverSocket)
 {
@@ -236,7 +236,7 @@ int ForkServer::forkServerLoop (UnixSocket *serverSocket)
           Socket socket = serverSocket->accept ();
 
           char command;
-          u_long nbr;
+          size_t nbr;
 
           if (socket.read (&command, 1, &nbr))
             {
@@ -247,7 +247,7 @@ int ForkServer::forkServerLoop (UnixSocket *serverSocket)
             {
             case 'e': //exit process
               socket.close ();
-              serverSocket->shutdown ();
+              serverSocket->shutdown (SHUT_RDWR);
               serverSocket->close ();
               exit (0);
               return 0;
@@ -274,13 +274,13 @@ int ForkServer::forkServerLoop (UnixSocket *serverSocket)
 }
 
 /*!
- *Execute a process using the fork server.
- *\param spi New process information.
- *\param flags Flags.
- *\param pid The new process ID.
- *\param port if FLAG_STDIN_SOCKET was specified.
- *\param waitEnd If true `executeProcess' will wait until
- *the process terminates.
+  Execute a process using the fork server.
+  \param spi New process information.
+  \param flags Flags.
+  \param pid The new process ID.
+  \param port if FLAG_STDIN_SOCKET was specified.
+  \param waitEnd If true `executeProcess' will wait until
+  the process terminates.
  */
 int ForkServer::executeProcess (StartProcInfo *spi,
                                 int flags,
@@ -291,7 +291,7 @@ int ForkServer::executeProcess (StartProcInfo *spi,
 #ifdef WIN32
   return 0;
 #else
-  u_long nbw;
+  size_t nbw;
   int len = 0;
   const char * env = (const char *) spi->envString;
 
@@ -299,7 +299,7 @@ int ForkServer::executeProcess (StartProcInfo *spi,
     {
       UnixSocket sock;
       sock.socket ();
-      sock.connect (socketPath.c_str ());
+      sock.connect2 (socketPath.c_str ());
       sock.write ("r", 1, &nbw);
 
       writeInt (&sock, flags);
@@ -335,9 +335,7 @@ int ForkServer::executeProcess (StartProcInfo *spi,
     }
 
   if (waitEnd)
-    {
-      return waitpid (*pid, NULL, 0);
-    }
+    return gnulib::waitpid (*pid, NULL, 0);
 
   return 0;
 #endif
@@ -345,22 +343,29 @@ int ForkServer::executeProcess (StartProcInfo *spi,
 }
 
 /*!
- *Terminate the fork server execution.
+  Terminate the fork server execution.
  */
 void ForkServer::killServer ()
 {
-  u_long nbw;
+  size_t nbw;
   UnixSocket s;
-  s.socket ();
-  s.connect (socketPath.c_str ());
-  s.write ("e", 1, &nbw);
-  s.close ();
+  try
+    {
+      s.socket ();
+      s.connect2 (socketPath.c_str ());
+      s.write ("e", 1, &nbw);
+      s.close ();
+    }
+  catch (...)
+    {
+
+    }
 }
 
 /*!
- *Initialize the fork server.
- *
- *\return 0 on success.
+  Initialize the fork server.
+
+  \return 0 on success.
  */
 int ForkServer::startForkServer ()
 {
@@ -392,10 +397,10 @@ int ForkServer::startForkServer ()
 }
 
 /*!
- *Create a listener without specify a port.
- *
- *\param socket The socket to generate.
- *\param port the obtained port.
+  Create a listener without specify a port.
+
+  \param socket The socket to generate.
+  \param port the obtained port.
  */
 int ForkServer::generateListenerSocket (Socket &socket, u_short *port)
 {
@@ -404,7 +409,7 @@ int ForkServer::generateListenerSocket (Socket &socket, u_short *port)
 
   socket.socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
-  if (socket.getHandle () == (Handle)INVALID_SOCKET)
+  if (socket.getHandle () < 0)
     return -1;
 
   MYSERVER_SOCKADDR_STORAGE sockaddr = { 0 };
@@ -420,8 +425,9 @@ int ForkServer::generateListenerSocket (Socket &socket, u_short *port)
 
   *port = ntohs (((sockaddr_in*)(&sockaddr))->sin_port);
 
-  if (socket.setsockopt (SOL_SOCKET, SO_REUSEADDR, (const char *)&optvalReuseAddr,
-                        sizeof (optvalReuseAddr)) < 0)
+  if (socket.setsockopt (SOL_SOCKET, SO_REUSEADDR,
+                         (const char *) &optvalReuseAddr,
+                         sizeof (optvalReuseAddr)) < 0)
     return -1;
 
   return 0;
