@@ -1,7 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free
-  Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+  Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -16,7 +16,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 
 #include <include/conf/security/auth_domain.h>
 #include <include/conf/security/security_manager.h>
@@ -34,9 +34,9 @@ AuthDomain::~AuthDomain ()
 }
 
 /*!
- *\see SecurityDomain::getValue.
+  \see SecurityDomain::getValue.
  */
-string *AuthDomain::getValue (string &name)
+const string *AuthDomain::getValue (string &name)
 {
   if (!name.compare ("user"))
     return &(securityToken->getUser ());
