@@ -1,6 +1,6 @@
 /*
   MyServer
-  Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,7 +15,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 
 #include <include/conf/vhost/vhost_manager.h>
 #include <include/conf/vhost/vhost.h>
@@ -39,18 +39,13 @@ Vhost* VhostManagerHandler::getVHost (const char*, const char*, u_short)
   return NULL;
 }
 
-Vhost* VhostManagerHandler::getVHostByNumber (int n)
-{
-  return 0;
-}
-
-int VhostManagerHandler::addVHost (Vhost*)
+Vhost* VhostManagerHandler::getVHost (u_long n)
 {
   return 0;
 }
 
 /*!
- * C'tor.
+  C'tor.
  */
 VhostManager::VhostManager ()
 {
@@ -58,9 +53,9 @@ VhostManager::VhostManager ()
 }
 
 /*!
- * Set the handler where delegate all requests.
- *
- *\param handler The new handler.
+  Set the handler where delegate all requests.
+
+  \param handler The new handler.
  */
 void VhostManager::setHandler (VhostManagerHandler *handler)
 {
@@ -68,11 +63,11 @@ void VhostManager::setHandler (VhostManagerHandler *handler)
 }
 
 /*!
- *Get the vhost for the connection. A return value of 0 means that
- *a valid host was not found.
- *\param host Hostname for the virtual host.
- *\param ip IP address for the virtual host.
- *\param port The port used by the client to connect to the server.
+  Get the vhost for the connection. A return value of 0 means that
+  a valid host was not found.
+  \param host Hostname for the virtual host.
+  \param ip IP address for the virtual host.
+  \param port The port used by the client to connect to the server.
  */
 Vhost* VhostManager::getVHost (const char *host, const char *ip, u_short port)
 {
@@ -84,35 +79,36 @@ Vhost* VhostManager::getVHost (const char *host, const char *ip, u_short port)
 
 
 /*!
- *Get a virtual host by its position in the list.
- *Zero based list.
- *\param n The virtual host id.
+  Get a virtual host by its position in the list.
+  Zero based list.
+  \param n The virtual host id.
  */
-Vhost* VhostManager::getVHostByNumber (int n)
+Vhost* VhostManager::getVHost (int n)
 {
   if (handler)
-    return handler->getVHostByNumber (n);
+    return handler->getVHost (n);
 
   return NULL;
 }
 
 /*!
- * Register a builder function for a vhost manager.
- * \param name manager name.
- * \param builder Builder routine.
+  Register a builder function for a vhost manager.
+  \param name manager name.
+  \param builder Builder routine.
  */
-void VhostManager::registerBuilder (string &name, MAKE_HANDLER builder)
+void VhostManager::registerBuilder (const string &name, MAKE_HANDLER builder)
 {
-  builders.put (name, builder);
+  string nameStr (name);
+  builders.put (nameStr, builder);
 }
 
 /*!
- * Build an handler given its name.
- *
- * \param name handler name.
- * \return an instance of the requested handler type.
+  Build an handler given its name.
+
+  \param name handler name.
+  \return an instance of the requested handler type.
  */
-VhostManagerHandler *VhostManager::buildHandler (string &name,
+VhostManagerHandler *VhostManager::buildHandler (const string &name,
                                                  ListenThreads *lt,
                                                  LogManager *lm)
 {
