@@ -1,6 +1,6 @@
 /*
   MyServer
-  Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,7 +15,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 
 #include <include/server/server.h>
 #include <include/base/file/files_utility.h>
@@ -23,8 +23,6 @@
 #include <include/base/sync/semaphore.h>
 #include <include/base/sync/event.h>
 
-extern "C"
-{
 #ifdef WIN32
 # include <direct.h>
 #endif
@@ -40,7 +38,7 @@ extern "C"
 #  include <pthread.h>
 # endif
 #endif
-}
+
 
 #include <set>
 #include <algorithm>
@@ -49,7 +47,7 @@ extern "C"
 using namespace std;
 
 /*!
- * C'tor.
+  C'tor.
  */
 ListenThreads::ListenThreads (ConnectionsScheduler* scheduler, Server* server)
 {
@@ -60,12 +58,11 @@ ListenThreads::ListenThreads (ConnectionsScheduler* scheduler, Server* server)
 }
 
 /*!
- * This function is used to create a socket server and a thread listener
- * for a port.
+  This function is used to create a socket server and a thread listener
+  for a port.
  */
 int ListenThreads::createServerAndListener (u_short port)
 {
-  int optvalReuseAddr = 1;
   string listenPortMsg;
 
   if (fastRebooting)
@@ -75,8 +72,8 @@ int ListenThreads::createServerAndListener (u_short port)
     }
 
   /*
-   * Create the server sockets:
-   * one server socket for IPv4 and another one for IPv6
+    Create the server sockets:
+    one server socket for IPv4 and another one for IPv6
    */
   Socket *serverSocketIPv4 = new Socket ();
   Socket *serverSocketIPv6 = NULL;
@@ -85,178 +82,89 @@ int ListenThreads::createServerAndListener (u_short port)
   si->port = port;
 
   /*
-   * Create the server socket.
+    Create the server socket.
    */
   try
     {
-      if (serverSocketIPv4 != NULL)
-        {
-          serverSocketIPv4->socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
-          if (serverSocketIPv4->getHandle () == (Handle)INVALID_SOCKET)
-            {
-              server->log (MYSERVER_LOG_MSG_ERROR,
-                            _("Error while creating the server socket"));
-              delete serverSocketIPv4;
-              serverSocketIPv4 = NULL;
-            }
-          else
-            {
-              MYSERVER_SOCKADDR_STORAGE sockServerSocketIPv4 = { 0 };
-              ((sockaddr_in*)(&sockServerSocketIPv4))->sin_family = AF_INET;
-              ((sockaddr_in*)(&sockServerSocketIPv4))->sin_addr.s_addr =
-                htonl (INADDR_ANY);
-              ((sockaddr_in*)(&sockServerSocketIPv4))->sin_port =
-                htons ((u_short)port);
+      MYSERVER_SOCKADDR_STORAGE sockServerSocketIPv4 = { 0 };
+      sockaddr_in* sai = (sockaddr_in *) &sockServerSocketIPv4;
+      serverSocketIPv4->socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
+      sai->sin_family = AF_INET;
+      sai->sin_addr.s_addr = htonl (INADDR_ANY);
+      sai->sin_port = htons ((u_short) port);
+      serverSocketIPv4->reuseAddress (true);
+      server->log (MYSERVER_LOG_MSG_INFO, _("Binding the port"));
+      serverSocketIPv4->bind (&sockServerSocketIPv4, sizeof (sockaddr_in));
+      server->log (MYSERVER_LOG_MSG_INFO, _("Port was bound"));
 
-#ifndef WIN32
-              if (serverSocketIPv4->setsockopt (SOL_SOCKET, SO_REUSEADDR,
-                                                (const char *)&optvalReuseAddr,
-                                                sizeof (optvalReuseAddr)) < 0)
-                {
-                  server->log (MYSERVER_LOG_MSG_ERROR,
-                                 _("Error while creating the server socket"));
-
-                  delete serverSocketIPv4;
-                  serverSocketIPv4 = NULL;
-                }
-#endif
-              if (serverSocketIPv4)
-                {
-                  server->log (MYSERVER_LOG_MSG_INFO,
-                                      _("Binding the port"));
-
-                  if (!serverSocketIPv4->bind (&sockServerSocketIPv4,
-                                              sizeof (sockaddr_in)))
-                    server->log (MYSERVER_LOG_MSG_INFO, _("Port was bound"));
-                  else
-                    {
-                      server->log (MYSERVER_LOG_MSG_ERROR,
-                                          _("Error while creating the server socket"));
-                      delete serverSocketIPv4;
-                      serverSocketIPv4 = NULL;
-                    }
-
-                }
-            }
-        }
+      serverSocketIPv4->listen (SOMAXCONN);
+    }
+  catch (exception & e)
+    {
+      server->log (MYSERVER_LOG_MSG_ERROR,
+                   _E ("Error while creating the server socket"), &e);
+      delete serverSocketIPv4;
+      serverSocketIPv4 = NULL;
+    }
 
 #if HAVE_IPV6
-      serverSocketIPv6 = new Socket ();
+  serverSocketIPv6 = new Socket ();
+  try
+    {
+      MYSERVER_SOCKADDR_STORAGE sockServerSocketIPv6 = { 0 };
+      sockaddr_in6 *sai = (sockaddr_in6 *)(&sockServerSocketIPv6);
+      serverSocketIPv6->socket (AF_INET6, SOCK_STREAM, IPPROTO_TCP);
+      sai->sin6_family = AF_INET6;
+      sai->sin6_addr = in6addr_any;
+      sai->sin6_port = htons ((u_short) port);
+      serverSocketIPv6->reuseAddress (true);
 
-      if (serverSocketIPv6 != NULL)
-        {
-          serverSocketIPv6->socket (AF_INET6, SOCK_STREAM, IPPROTO_TCP);
-          if (serverSocketIPv6->getHandle () == (FileHandle)INVALID_SOCKET)
-            {
-              server->log (MYSERVER_LOG_MSG_ERROR,
-                                 _("Error while creating the server socket"));
-              delete serverSocketIPv6;
-              serverSocketIPv6 = NULL;
-            }
-          else
-            {
-              MYSERVER_SOCKADDR_STORAGE sockServerSocketIPv6 = { 0 };
-              ((sockaddr_in6*)(&sockServerSocketIPv6))->sin6_family = AF_INET6;
-              ((sockaddr_in6*)(&sockServerSocketIPv6))->sin6_addr = in6addr_any;
-              ((sockaddr_in6*)(&sockServerSocketIPv6))->sin6_port =
-                htons ((u_short)port);
 # ifndef WIN32
-              if (serverSocketIPv6->setsockopt (SOL_SOCKET, SO_REUSEADDR,
-                                                (const char *)&optvalReuseAddr,
-                                                sizeof (optvalReuseAddr)) < 0)
-                {
-                  server->log (MYSERVER_LOG_MSG_ERROR,
-                                 _("Error while creating the server socket"));
-                  delete serverSocketIPv6;
-                  serverSocketIPv6 = NULL;
-                }
-
-              if (serverSocketIPv6->setsockopt (IPPROTO_IPV6, IPV6_V6ONLY,
-                                              (const char *)&optvalReuseAddr,
-                                              sizeof (optvalReuseAddr)) < 0)
-                {
-                  server->log (MYSERVER_LOG_MSG_ERROR,
-                                 _("Error while creating the server socket"));
-                  delete serverSocketIPv6;
-                  serverSocketIPv6 = NULL;
-                }
+      int one = 1;
+      serverSocketIPv6->setsockopt (IPPROTO_IPV6, IPV6_V6ONLY,
+                                    (const char *) &one,
+                                    sizeof (one));
 # endif
-              if (serverSocketIPv6)
-                {
-                  server->log (MYSERVER_LOG_MSG_INFO,
-                                      _("Binding the port"));
-
-                  if (!serverSocketIPv6->bind (&sockServerSocketIPv6,
-                                               sizeof (sockaddr_in6)))
-                    server->log (MYSERVER_LOG_MSG_INFO,
-                                        _("Port was bound"));
-                  else
-                    {
-                      server->log (MYSERVER_LOG_MSG_ERROR,
-                                 _("Error while creating the server socket"));
-                      delete serverSocketIPv6;
-                      serverSocketIPv6 = NULL;
-                    }
-                }
-            }
-        }
-#endif // HAVE_IPV6
-
-      if (serverSocketIPv4 == NULL && serverSocketIPv6 == NULL)
-        {
-          delete si;
-          return 1;
-        }
-
-      if (serverSocketIPv4 != NULL && serverSocketIPv4->listen (SOMAXCONN))
-        {
-          server->log (MYSERVER_LOG_MSG_ERROR,
-                             _("Error while creating the server socket"));
-          delete serverSocketIPv4;
-          serverSocketIPv4 = NULL;
-        }
-
-      if (serverSocketIPv6 != NULL && serverSocketIPv6->listen (SOMAXCONN))
-        {
-          server->log (MYSERVER_LOG_MSG_ERROR,
-                             _("Error while creating the server socket"));
-          delete serverSocketIPv6;
-          serverSocketIPv6 = NULL;
-        }
-
-      if (serverSocketIPv4 == NULL && serverSocketIPv6 == NULL)
-        {
-          delete si;
-          return 1;
-        }
-
-      server->log (MYSERVER_LOG_MSG_INFO,
-                          _("Listening on the port: %i"), port);
-
-      si->ipv4 = serverSocketIPv4;
-      si->ipv6 = serverSocketIPv6;
-
-      usedPorts.put (port, si);
-
-      registerListener (si);
-
-      return 0;
+      server->log (MYSERVER_LOG_MSG_INFO, _("Binding the port"));
+      serverSocketIPv6->bind (&sockServerSocketIPv6,  sizeof (sockaddr_in6));
+      server->log (MYSERVER_LOG_MSG_INFO, _("Port was bound"));
+      serverSocketIPv6->listen (SOMAXCONN);
     }
-  catch (bad_alloc &ba)
+  catch (exception & e)
     {
-      server->log (MYSERVER_LOG_MSG_ERROR, _("Bad alloc: %s"),
-                          ba.what ());
+      server->log (MYSERVER_LOG_MSG_ERROR,
+                   _("Error while creating the IPv6 server socket"), &e);
+      delete serverSocketIPv6;
+      serverSocketIPv6 = NULL;
     }
-  catch (exception &e)
+#endif
+
+  if (serverSocketIPv4 == NULL && serverSocketIPv6 == NULL)
     {
-      server->log (MYSERVER_LOG_MSG_ERROR, _("Error: %s"),
-                          e.what ());
-    };
-  return 1;
+      delete si;
+      return 1;
+    }
+
+  if (serverSocketIPv4 == NULL && serverSocketIPv6 == NULL)
+    {
+      delete si;
+      return 1;
+    }
+
+  server->log (MYSERVER_LOG_MSG_INFO, _("Listening on the port: %i"), port);
+
+  si->ipv4 = serverSocketIPv4;
+  si->ipv6 = serverSocketIPv6;
+
+  usedPorts.put (port, si);
+
+  registerListener (si);
+
+  return 0;
 }
 
 /*!
- * Register the sockets on the events listener.
+  Register the sockets on the events listener.
  */
 void ListenThreads::registerListener (SocketInformation* si)
 {
@@ -266,16 +174,16 @@ void ListenThreads::registerListener (SocketInformation* si)
       scheduler->listener (&(si->laIpv4));
     }
 
-    if (si->ipv6)
-      {
-        si->laIpv6.reset (si->ipv6, si->port, server);
-        scheduler->listener (&(si->laIpv6));
-      }
+  if (si->ipv6)
+    {
+      si->laIpv6.reset (si->ipv6, si->port, server);
+      scheduler->listener (&(si->laIpv6));
+    }
 }
 
 /*!
- * Add a listening thread on a specific port.
- * \param port Port to listen on.
+  Add a listening thread on a specific port.
+  \param port Port to listen on.
  */
 void ListenThreads::addListeningThread (u_short port)
 {
@@ -286,8 +194,8 @@ void ListenThreads::addListeningThread (u_short port)
 }
 
 /*!
- * Initialize the listen threads manager.
- * \param parser Xml data to use for error messages.
+  Initialize the listen threads manager.
+  \param parser Xml data to use for error messages.
  */
 int ListenThreads::initialize ()
 {
@@ -296,7 +204,7 @@ int ListenThreads::initialize ()
 }
 
 /*!
- * Complete the fast reboot.
+  Complete the fast reboot.
  */
 void ListenThreads::commitFastReboot ()
 {
@@ -343,8 +251,6 @@ void ListenThreads::commitFastReboot ()
   set_difference (newPorts.begin (), newPorts.end (), presentPorts.begin (),
     presentPorts.end (), insert_iterator<set<u_short> >(toAdd, toAdd.begin ()));
 
-
-
   /* Ports in intersections need only to be registered on the event listener.  */
   for (set<u_short>::iterator it = intersection.begin ();
        it != intersection.end (); it++)
@@ -373,7 +279,7 @@ void ListenThreads::commitFastReboot ()
 
 
 /*!
- * Restore the previous situation without do anything.
+  Restore the previous situation without do anything.
  */
 void ListenThreads::rollbackFastReboot ()
 {
@@ -389,7 +295,7 @@ void ListenThreads::rollbackFastReboot ()
 }
 
 /*!
- * Prepare the listen threads manager for a fast reboot.
+  Prepare the listen threads manager for a fast reboot.
  */
 void ListenThreads::beginFastReboot ()
 {
@@ -397,7 +303,7 @@ void ListenThreads::beginFastReboot ()
 }
 
 /*!
- * Unload the listen threads manager.
+  Unload the listen threads manager.
  */
 int ListenThreads::terminate ()
 {
@@ -440,11 +346,18 @@ int ListenThreads::terminate ()
           if (!serverSocket)
             continue;
 
-          serverSocket->shutdown (SD_BOTH);
-          do
+          serverSocket->shutdown (SHUT_RDWR);
+          for (;;)
             {
-              err = serverSocket->recv (buffer, 256, 0);
-            }while (err != -1);
+              try
+                {
+                  err = serverSocket->recv (buffer, 256, 0);
+                }
+              catch (...)
+                {
+                  break;
+                }
+            }
 
           serverSocket->close ();
           delete serverSocket;
