@@ -1,6 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009, 2010 Free Software
+  Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,11 +16,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "myserver.h"
 #include <include/filter/gzip/gzip_decompress.h>
 #include <include/base/string/securestr.h>
 
-extern "C"
-{
 #ifdef WIN32
 # include <direct.h>
 # include <errno.h>
@@ -27,13 +27,13 @@ extern "C"
 # include <string.h>
 # include <errno.h>
 #endif
-}
+
 #ifdef WIN32
 # include <algorithm>
 #endif
 
 /*!
- * Initialize the gzip structure value.
+  Initialize the gzip structure value.
  */
 u_long GzipDecompress::initialize ()
 {
@@ -54,14 +54,14 @@ u_long GzipDecompress::initialize ()
 }
 
 /*!
- * Decompress the in buffer to the out buffer using the gzip compression.
- * \param in Buffer to decompress.
- * \param sizeIn The dimension of the buffer to decompress.
- * \param out Buffer where decompress.
- * \param sizeOut The dimension of the buffer where decompress.
+  Decompress the in buffer to the out buffer using the gzip compression.
+  \param in Buffer to decompress.
+  \param sizeIn The dimension of the buffer to decompress.
+  \param out Buffer where decompress.
+  \param sizeOut The dimension of the buffer where decompress.
  */
-u_long GzipDecompress::decompress (const char* in, u_long sizeIn,
-                                   char *out, u_long sizeOut)
+u_long GzipDecompress::decompress (const char* in, size_t sizeIn,
+                                   char *out, size_t sizeOut)
 {
 #ifdef HAVE_ZLIB
   u_long old_total_out = data.stream.total_out;
@@ -72,7 +72,7 @@ u_long GzipDecompress::decompress (const char* in, u_long sizeIn,
   data.stream.next_out = (Bytef*) out;
   data.stream.avail_out = sizeOut;
 
-  ret = inflate(&(data.stream), Z_FULL_FLUSH);
+  ret = inflate (&(data.stream), Z_FULL_FLUSH);
 
   data.data_size += data.stream.total_out - old_total_out;
   return data.stream.total_out - old_total_out;
@@ -83,40 +83,39 @@ u_long GzipDecompress::decompress (const char* in, u_long sizeIn,
 }
 
 /*!
- * Close the gzip compression.
+  Close the gzip compression.
  */
 u_long GzipDecompress::free ()
 {
   u_long ret = 0;
 #ifdef HAVE_ZLIB
-
   if (data.initialized == 0)
     return 0;
   data.initialized = 0;
-  ret = inflateEnd(&(data.stream));
+  ret = inflateEnd (&(data.stream));
 #endif
   return ret;
 }
 
 /*!
- * Inherited from Filter.
- * \param buffer Buffer where write.
- * \param len Buffer length.
- * \param nbw Numbers of written bytes.
+  Inherited from Filter.
+  \param buffer Buffer where write.
+  \param len Buffer length.
+  \param nbw Numbers of written bytes.
  */
-int GzipDecompress::getHeader (char* buffer, u_long len, u_long* nbw)
+int GzipDecompress::getHeader (char* buffer, size_t len, size_t* nbw)
 {
   *nbw = getHeader (buffer, len);
   return !(*nbw);
 }
 
 /*!
- * Inherited from Filter.
- * \param buffer Buffer where write.
- * \param len Buffer length.
- * \param nbw Numbers of written bytes.
+  Inherited from Filter.
+  \param buffer Buffer where write.
+  \param len Buffer length.
+  \param nbw Numbers of written bytes.
  */
-int GzipDecompress::getFooter (char* buffer, u_long len, u_long* nbw)
+int GzipDecompress::getFooter (char* buffer, size_t len, size_t* nbw)
 {
   if (len < GZIP_FOOTER_LENGTH)
     return -1;
@@ -126,19 +125,19 @@ int GzipDecompress::getFooter (char* buffer, u_long len, u_long* nbw)
 }
 
 /*!
- * The GzipDecompress filter modifies the data.
+  The GzipDecompress filter modifies the data.
  */
-int GzipDecompress::modifyData()
+int GzipDecompress::modifyData ()
 {
   return 1;
 }
 
 /*!
- * Flush all the remaining data.
- * \param out Buffer where write.
- * \param sizeOut Buffer length.
+  Flush all the remaining data.
+  \param out Buffer where write.
+  \param sizeOut Buffer length.
  */
-u_long GzipDecompress::flush (char *out, u_long sizeOut)
+u_long GzipDecompress::flush (char *out, size_t sizeOut)
 {
 #ifdef HAVE_ZLIB
   u_long old_total_out = data.stream.total_out;
@@ -149,7 +148,7 @@ u_long GzipDecompress::flush (char *out, u_long sizeOut)
   data.stream.avail_in = 0;
   data.stream.next_out = (Bytef*) out;
   data.stream.avail_out = destLen;
-  inflate(&(data.stream), Z_FINISH);
+  inflate (&(data.stream), Z_FINISH);
 
   data.data_size += data.stream.total_out - old_total_out;
   return data.stream.total_out - old_total_out;
@@ -159,7 +158,7 @@ u_long GzipDecompress::flush (char *out, u_long sizeOut)
 }
 
 /*!
- * Constructor for the class.
+  Constructor for the class.
  */
 GzipDecompress::GzipDecompress ()
 {
@@ -168,19 +167,19 @@ GzipDecompress::GzipDecompress ()
 }
 
 /*!
- * Destructor for the class.
+  Destructor for the class.
  */
-GzipDecompress::~GzipDecompress()
+GzipDecompress::~GzipDecompress ()
 {
-  free();
+  free ();
 }
 
 /*!
- * Get the GZIP footer.
- * \param str Buffer where write.
- * \param size Buffer length.
+  Get the GZIP footer.
+  \param str Buffer where write.
+  \param size Buffer length.
  */
-u_long GzipDecompress::getFooter(char *str, int /*size*/)
+u_long GzipDecompress::getFooter (char *str, size_t /*size*/)
 {
 #ifdef HAVE_ZLIB
   return GZIP_FOOTER_LENGTH;
@@ -190,61 +189,63 @@ u_long GzipDecompress::getFooter(char *str, int /*size*/)
 }
 
 /*!
- * Copy the GZIP header in the buffer.
- * \param buffer Buffer where write.
- * \param buffersize Buffer length.
+  Copy the GZIP header in the buffer.
+  \param buffer Buffer where write.
+  \param buffersize Buffer length.
  */
-u_long GzipDecompress::getHeader (char *buffer, u_long buffersize)
+u_long GzipDecompress::getHeader (char *buffer, size_t buffersize)
 {
+#ifdef HAVE_ZLIB
   if (buffersize < GZIP_HEADER_LENGTH)
     return 0;
   memcpy (buffer, GZIP_HEADER, GZIP_HEADER_LENGTH);
   return GZIP_HEADER_LENGTH;
+#else
+  return 0;
+#endif
 }
 
 /*!
- * Inherited from Filter.
- * This function uses an internal buffer slowing it.
- * It is better to use directly the GzipDecompress::compress routine where possible.
- * \param buffer Buffer where write.
- * \param len Buffer length.
- * \param nbr Number of read bytes.
+  Inherited from Filter.
+  This function uses an internal buffer slowing it.
+  It is better to use directly the GzipDecompress::compress routine where possible.
+  \param buffer Buffer where write.
+  \param len Buffer length.
+  \param nbr Number of read bytes.
  */
-int GzipDecompress::read (char* buffer, u_long len, u_long *nbr)
+int GzipDecompress::read (char* buffer, size_t len, size_t *nbr)
 {
   char *tmp_buff;
-  int ret;
-  u_long nbr_parent;
-  if(!parent)
-    return -1;
-  tmp_buff = new char[len/2];
-  if (!tmp_buff)
+  size_t nbr_parent;
+  if (! parent)
     return -1;
 
-  if (!active)
+  if (! active)
     return parent->read (buffer, len, nbr);
 
-  ret = parent->read (tmp_buff, len/2, &nbr_parent);
-
-  if (ret == -1)
+  tmp_buff = new char[len/2];
+  try
+    {
+      parent->read (tmp_buff, len/2, &nbr_parent);
+      *nbr = decompress (tmp_buff, nbr_parent, buffer, len);
+    }
+  catch (...)
     {
       delete [] tmp_buff;
-      return -1;
+      throw;
     }
 
-  *nbr = decompress (tmp_buff, nbr_parent, buffer, len);
-
   delete [] tmp_buff;
   return 0;
 }
 
 /*!
- * Inherited from Filter.
- * \param buffer Buffer where write.
- * \param len Buffer length.
- * \param nbw Number of written bytes.
+  Inherited from Filter.
+  \param buffer Buffer where write.
+  \param len Buffer length.
+  \param nbw Number of written bytes.
  */
-int GzipDecompress::write (const char* buffer, u_long len, u_long *nbw)
+int GzipDecompress::write (const char* buffer, size_t len, size_t *nbw)
 {
   char tmpBuffer[1024];
   u_long written = 0;
@@ -259,8 +260,8 @@ int GzipDecompress::write (const char* buffer, u_long len, u_long *nbw)
 
   while (len)
     {
-      u_long nbw_parent;
-      u_long size = std::min (len, 512UL);
+      size_t nbw_parent;
+      size_t size = std::min (len, (size_t) 512);
       u_long ret = decompress (buffer, size, tmpBuffer, 1024);
 
       if (ret && parent->write (tmpBuffer, ret, &nbw_parent) == -1)
@@ -275,21 +276,21 @@ int GzipDecompress::write (const char* buffer, u_long len, u_long *nbw)
 }
 
 /*!
- * Inherited from Filter.
- * \param nbw Number of flushed bytes.
+  Inherited from Filter.
+  \param nbw Number of flushed bytes.
  */
-int GzipDecompress::flush (u_long *nbw)
+int GzipDecompress::flush (size_t *nbw)
 {
   char buffer[512];
 
-  if(!active)
+  if (!active)
     return 0;
 
   *nbw = flush (buffer, 512);
   if (*nbw)
     {
-      u_long nbwParent;
-      u_long nbwParentFlush;
+      size_t nbwParent;
+      size_t nbwParentFlush;
       if (!parent)
         return -1;
       if (parent->write (buffer, *nbw, &nbwParent) != 0)
@@ -302,8 +303,8 @@ int GzipDecompress::flush (u_long *nbw)
 }
 
 /*!
- * Returns a new GzipDecompress object.
- * \param name Filter name.
+  Returns a new GzipDecompress object.
+  \param name Filter name.
  */
 Filter* GzipDecompress::factory (const char* name)
 {
@@ -311,32 +312,28 @@ Filter* GzipDecompress::factory (const char* name)
 }
 
 /*!
- * Return a string with the filter name.
- * If an external buffer is provided write the name there too.
- * \param name Buffer where write the filter name.
- * \param len Buffer size.
+  Return a string with the filter name.
+  If an external buffer is provided write the name there too.
+  \param name Buffer where write the filter name.
+  \param len Buffer size.
  */
-const char* GzipDecompress::getName (char* name, u_long len)
+const char* GzipDecompress::getName ()
 {
-  /*! No name by default. */
-  if (name)
-    myserver_strlcpy(name, "gzip", len);
-
   return "gzip";
 }
 
 /*!
- * Get the GZIP header size.
+  Get the GZIP header size.
  */
-u_long GzipDecompress::headerSize()
+u_long GzipDecompress::headerSize ()
 {
   return GZIP_HEADER_LENGTH;
 }
 
 /*!
- * Get the GZIP footer size.
+  Get the GZIP footer size.
  */
-u_long GzipDecompress::footerSize()
+u_long GzipDecompress::footerSize ()
 {
   return GZIP_FOOTER_LENGTH;
 }
