@@ -1,6 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2005, 2006, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2005, 2006, 2008, 2009, 2010 Free Software Foundation,
+  Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,7 +16,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 
 #include <include/protocol/http/dyn_http_command.h>
 #include <include/server/server.h>
@@ -23,7 +24,7 @@
 
 
 /*!
- *Default constructor.
+  Default constructor.
  */
 DynamicHttpCommand::DynamicHttpCommand (string& name)
 {
@@ -31,7 +32,7 @@ DynamicHttpCommand::DynamicHttpCommand (string& name)
 }
 
 /*!
- *Default constructor.
+  Default constructor.
  */
 string DynamicHttpCommand::getName ()
 {
@@ -39,7 +40,7 @@ string DynamicHttpCommand::getName ()
 }
 
 /*!
- *Destroy the object.
+  Destroy the object.
  */
 DynamicHttpCommand::~DynamicHttpCommand ()
 {
