@@ -1,6 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
+  Copyright (C) 2005, 2007, 2008, 2009, 2010 Free Software Foundation,
+  Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,7 +16,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 #include <include/protocol/http/http_response.h>
 #include <include/base/string/stringutils.h>
 #include <iostream>
@@ -23,7 +24,7 @@
 using namespace std;
 
 /*!
- *Create the object.
+  Create the object.
  */
 HttpResponseHeader::HttpResponseHeader ()
 {
@@ -32,7 +33,7 @@ HttpResponseHeader::HttpResponseHeader ()
 
 
 /*!
- *Destroy the object.
+  Destroy the object.
  */
 HttpResponseHeader::~HttpResponseHeader ()
 {
@@ -42,7 +43,7 @@ HttpResponseHeader::~HttpResponseHeader ()
 }
 
 /*!
- *Reset the object.
+  Reset the object.
  */
 void HttpResponseHeader::free ()
 {
@@ -58,73 +59,50 @@ void HttpResponseHeader::free ()
 }
 
 /*!
- *Get the value of the [name] field.
+  Get the value of the [name] field.
  */
 string* HttpResponseHeader::getValue (const char* name, string* out)
 {
-  if (!strcmpi (name, "Ver"))
+  if (! strcasecmp (name, "Ver"))
     {
       if (out)
         out->assign (ver.c_str ());
       return &ver;
     }
 
-  if (!strcmpi (name, "Content-length"))
+  if (! strcasecmp (name, "Content-length"))
     {
       if (out)
         out->assign (contentLength.c_str ());
       return &contentLength;
     }
 
-  HttpResponseHeader::Entry *e = other.get (name);
-  if (e)
-    {
-      if (out)
-        out->assign (*(e->value));
-      return e->value;
-    }
-  return 0;
+  return HttpHeader::getValue (name, out);
 }
 
 /*!
- *Set the value of the [name] field to [in].
+  Set the value of the [name] field to [in].
  */
 string* HttpResponseHeader::setValue (const char* name, const char* in)
 {
-  if (!strcmpi (name, "Ver"))
+  if (! strcasecmp (name, "Ver"))
     {
       ver.assign (in);
       return &ver;
     }
 
-  if (!strcmpi (name, "Content-length"))
+  if (! strcasecmp (name, "Content-length"))
     {
       contentLength.assign (in);
       return &contentLength;
     }
 
-  {
-    HttpResponseHeader::Entry *e = other.get (name);
-    if (e)
-      {
-        e->value->assign (in);
-        return (e->value);
-      }
-    else
-      {
-        e = new HttpResponseHeader::Entry;
-        e->name->assign (name);
-        e->value->assign (in);
-        other.put (*e->name, e);
-      }
-
-    return 0;
-  }
+  return HttpHeader::setValue (name, in);
 }
 
 /*!
- *Get the kind of HTTP status code specified by the httpStatus variable.
- *\return the HTTP status kind.
+  Get the kind of HTTP status code specified by the httpStatus variable.
+  \return the HTTP status kind.
  */
 int HttpResponseHeader::getStatusType ()
 {
