@@ -1,6 +1,7 @@
 /*
   MyServer
-  Copyright (C) 2002-2009 Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+  Free Software Foundation, Inc.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
@@ -15,7 +16,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "stdafx.h"
+#include "myserver.h"
 #include <include/server/server.h>
 #include <include/server/clients_thread.h>
 #include <include/base/safetime/safetime.h>
@@ -36,12 +37,11 @@
 #include <include/base/crypt/sha1.h>
 
 #include <include/conf/main/xml_main_configuration.h>
+#include <include/conf/security/xml_validator.h>
 
 #include <cstdarg>
 
 
-extern "C"
-{
 #ifdef WIN32
 # include <direct.h>
 #else
@@ -58,17 +58,17 @@ extern "C"
 # endif
 
 #endif
-}
+
 #include <sstream>
 
 Server* Server::instance = NULL;
 
-Server::Server () : connectionsScheduler (this),
-                   listenThreads (&connectionsScheduler, this),
-                   authMethodFactory (&cryptAlgoManager),
-                   validatorFactory (),
-                   securityManager (&validatorFactory, &authMethodFactory),
-                   connectionsPool (100)
+Server::Server () : connectionsPool (100),
+                    connectionsScheduler (this),
+                    listenThreads (&connectionsScheduler, this),
+                    authMethodFactory (&cryptAlgoManager),
+                    validatorFactory (),
+                    securityManager (&validatorFactory, &authMethodFactory)
 {
   toReboot = false;
   autoRebootEnabled = true;
@@ -93,7 +93,7 @@ void
 Server::initLogManager ()
 {
   if (!logLocation.size ())
-    logLocation.assign ("console://stdout");
+    logLocation.assign ("console://stderr");
 
   if (!logManager->containsOwner (this))
     {
@@ -103,8 +103,8 @@ Server::initLogManager ()
 }
 
 /*!
- *Reinitialize the configuration paths, setting them to the specified ones.
- *Returns false on error.
+  Reinitialize the configuration paths, setting them to the specified ones.
+  Returns false on error.
  */
 bool Server::resetConfigurationPaths (string &mainConf, string &mimeConf,
 				     string &vhostConf, string &externPath)
@@ -117,10 +117,12 @@ bool Server::resetConfigurationPaths (string &mainConf, string &mimeConf,
 }
 
 /*!
- * Load here all the libraries.
+  Load here all the libraries.
  */
 int Server::loadLibraries ()
 {
+  /* Initialize the SSL library.  */
+  initializeSSL ();
   Process::initialize ();
   XmlParser::startXML ();
   myserver_safetime_init ();
@@ -136,7 +138,7 @@ int Server::loadLibraries ()
 }
 
 /*!
- * Destroy the object.
+  Destroy the object.
  */
 Server::~Server ()
 {
@@ -163,7 +165,7 @@ Server::~Server ()
 }
 
 /*!
- *Start the server.
+  Start the server.
  */
 void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
                     string &externPath, MainConfiguration* (*genMainConf)
@@ -203,10 +205,8 @@ void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
         return;
       }
 
-    /* Initialize the SSL library.  */
-    initializeSSL ();
-
-    log (MYSERVER_LOG_MSG_INFO, _("Loading server configuration..."));
+    log (MYSERVER_LOG_MSG_INFO, _("Loading server configuration from %s..."),
+         mainConf.c_str ());
 
     if (postLoad ())
       {
@@ -253,7 +253,7 @@ void Server::start (string &mainConf, string &mimeConf, string &vhostConf,
 }
 
 /*!
- * Complete the loading phase.
+  Complete the loading phase.
  */
 int Server::postLoad ()
 {
@@ -261,7 +261,7 @@ int Server::postLoad ()
   string buffer;
 
   /* Get the name of the local machine.  */
-  memset (serverName, 0, HOST_NAME_MAX+1);
+  memset (serverName, 0, HOST_NAME_MAX + 1);
   Socket::gethostname (serverName, HOST_NAME_MAX);
 
   log (MYSERVER_LOG_MSG_INFO, _("Host name: %s"), serverName);
@@ -271,7 +271,11 @@ int Server::postLoad ()
     delete ipAddresses;
   ipAddresses = new string ();
 
-  if (Socket::getLocalIPsList (*ipAddresses))
+  try
+    {
+      Socket::getLocalIPsList (*ipAddresses);
+    }
+  catch (exception & e)
     {
       log (MYSERVER_LOG_MSG_ERROR, _("Error reading IP list"));
       return -1;
@@ -303,7 +307,8 @@ int Server::postLoad ()
     return -1;
 
   /* Load the MIME types.  */
-  log (MYSERVER_LOG_MSG_INFO, _("Loading MIME types..."));
+  log (MYSERVER_LOG_MSG_INFO, _("Loading MIME types from %s..."),
+       mimeConfigurationFile.c_str ());
 
   string handlerStr (getData ("server.mime_handler", "xml"));
   mimeManager.setDefaultHandler (handlerStr);
@@ -342,7 +347,7 @@ int Server::postLoad ()
 }
 
 /*!
- * Reload the virtual hosts configuration.
+  Reload the virtual hosts configuration.
  */
 int Server::loadVHostConf ()
 {
@@ -360,8 +365,14 @@ int Server::loadVHostConf ()
 
   vhostManager.setHandler (vhostHandler);
 
+  const char *vhostLocation = getData ("server.vhost_location",
+                                       vhostConfigurationFile.c_str ());
+
   /* Load the virtual hosts configuration.  */
-  if (vhostHandler->load (vhostConfigurationFile.c_str ()))
+  log (MYSERVER_LOG_MSG_INFO, _("Loading virtual hosts from %s..."),
+       vhostLocation);
+
+  if (vhostHandler->load (vhostLocation))
     {
       log (MYSERVER_LOG_MSG_ERROR,
            _("Error loading the vhost configuration file %s"),
@@ -373,12 +384,13 @@ int Server::loadVHostConf ()
 }
 
 /*!
- * Load static built-in components.
+  Load static built-in components.
  */
 void Server::loadStaticComponents ()
 {
   string xml ("xml");
 
+  mimeManager.setLogger (this);
   mimeManager.registerHandler (xml, new XmlMimeHandler ());
 
   XmlVhostHandler::registerBuilder (vhostManager);
@@ -411,7 +423,7 @@ void Server::loadStaticComponents ()
 }
 
 /*!
- * Load the plugins.
+  Load the plugins.
  */
 void Server::loadPlugins ()
 {
@@ -421,7 +433,7 @@ void Server::loadPlugins ()
 }
 
 /*!
- * Server main loop.
+  Server main loop.
  */
 void Server::mainLoop ()
 {
@@ -436,16 +448,15 @@ void Server::mainLoop ()
   mimeConfTime = FilesUtility::getLastModTime (mimeConfigurationFile.c_str ());
 
   /*
-   * Keep thread alive.
-   * When the endServer flag is set to True exit
-   * from the loop and terminate the server execution.
+    Keep thread alive.
+    When the endServer flag is set to True exit
+    from the loop and terminate the server execution.
    */
   while (!endServer)
     {
-      Thread::wait (1000000);
+      Thread::wait (MYSERVER_SEC (1));
 
-      /* Check threads.  */
-      if (purgeThreadsCounter++ >= 10)
+      if (purgeThreadsCounter++ >= 120)
         {
           purgeThreadsCounter = 0;
           purgeThreads ();
@@ -453,6 +464,8 @@ void Server::mainLoop ()
 
       if (autoRebootEnabled)
         {
+          bool doBeep = false;
+
           time_t mainConfTimeNow =
             FilesUtility::getLastModTime (mainConfigurationFile.c_str ());
           time_t hostsConfTimeNow =
@@ -461,10 +474,10 @@ void Server::mainLoop ()
             FilesUtility::getLastModTime (mimeConfigurationFile.c_str ());
 
           /* If a configuration file was modified reboot the server. */
-          if (((mainConfTimeNow != -1) && (hostsConfTimeNow != -1)  &&
-               (mimeConfNow != -1)) || toReboot)
+          if (((mainConfTimeNow != -1) && (hostsConfTimeNow != -1)
+               && (mimeConfNow != -1)) || toReboot)
             {
-              if ( (mainConfTimeNow  != mainConfTime) || toReboot)
+              if ((mainConfTimeNow  != mainConfTime) || toReboot)
                 {
                   string msg ("main-conf-changed");
                   notifyMulticast (msg, 0);
@@ -479,13 +492,8 @@ void Server::mainLoop ()
                   string msg ("mime-conf-changed");
                   notifyMulticast (msg, 0);
 
-                  if (logLocation.find ("console://") != string::npos)
-                    {
-                      char beep[] = { static_cast<char>(0x7), '\0' };
-                      string beepStr (beep);
-                      logManager->log (this, "MAINLOG", logLocation, beepStr);
-                    }
-                  log (MYSERVER_LOG_MSG_INFO, _("Reloading MIME types"));
+                  log (MYSERVER_LOG_MSG_INFO, _("Reloading MIME types from %s"),
+                       mimeConfigurationFile.c_str ());
 
                   getMimeManager ()->reload ();
 
@@ -530,13 +538,20 @@ void Server::mainLoop ()
                   hostsConfTime = hostsConfTimeNow;
                   log (MYSERVER_LOG_MSG_INFO, _("Reloaded"));
                 }
+
+              if (doBeep && logLocation.find ("console://") != string::npos)
+                {
+                  char beep[] = { static_cast<char>(0x7), '\0' };
+                  string beepStr (beep);
+                  logManager->log (this, "MAINLOG", logLocation, beepStr);
+                }
             }
         }
     }
 }
 
 /*!
- * Display the MyServer boot.
+  Display the MyServer boot.
  */
 void Server::displayBoot ()
 {
@@ -556,14 +571,11 @@ void Server::displayBoot ()
 
 #endif /* CLEAR_BOOT_SCREEN.  */
 
-  /*
-   * Print the MyServer signature only if the log writes to the console.
-   */
+  /* Print the MyServer signature only if the log writes to the console.  */
   if (logLocation.find ("console://") != string::npos)
     {
       try
         {
-          size_t length;
           string softwareSignature = "************ GNU MyServer "
             MYSERVER_VERSION " ************";
 
@@ -590,15 +602,15 @@ void Server::displayBoot ()
 }
 
 /*!
- * Removed threads that can be destroyed.
- * The function returns the number of threads that were destroyed.
+  Removed threads that can be destroyed.
+  The function returns the number of threads that were destroyed.
  */
 int Server::purgeThreads ()
 {
   u_long ticks = getTicks ();
   u_long destroyed = 0;
 
-  purgeThreadsThreshold = std::min (purgeThreadsThreshold << 1,
+  purgeThreadsThreshold = std::min (purgeThreadsThreshold * 3 / 2,
                                     nMaxThreads);
   threadsMutex->lock ();
   for (list<ClientsThread*>::iterator it = threads.begin ();
@@ -606,21 +618,14 @@ int Server::purgeThreads ()
     {
       ClientsThread* thread = *it;
 
-
-      /*
-       *Shutdown all threads that can be destroyed.
-       */
       if (thread->isStopped ())
         {
           ClientsThread* thread = *it;
           list<ClientsThread*>::iterator next = it;
           next++;
 
-          thread->join ();
           threads.erase (it);
           delete thread;
-
-          destroyed++;
           it = next;
         }
       else if (thread->isToDestroy ())
@@ -628,14 +633,17 @@ int Server::purgeThreads ()
           if (destroyed < purgeThreadsThreshold)
             thread->stop ();
 
+          destroyed++;
           it++;
         }
       else
         {
-          if (!thread->isStatic ())
-            if (ticks - thread->getTimeout () > MYSERVER_SEC (15))
-              thread->setToDestroy (1);
-
+          if (! thread->isStatic ())
+            if (ticks - thread->getTimeout () > MYSERVER_SEC (20))
+              {
+                thread->setToDestroy (1);
+                destroyed++;
+              }
           it++;
         }
     }
@@ -645,7 +653,7 @@ int Server::purgeThreads ()
 }
 
 /*!
- * Do the final cleanup.  Called once when the process is terminated.
+  Do the final cleanup.  Called once when the process is terminated.
  */
 void Server::finalCleanup ()
 {
@@ -655,7 +663,7 @@ void Server::finalCleanup ()
 }
 
 /*!
- * Return the user identifier to use for the process.
+  Return the user identifier to use for the process.
  */
 const char *Server::getUid ()
 {
@@ -663,7 +671,7 @@ const char *Server::getUid ()
 }
 
 /*!
- * Return the group identifier to use for the process.
+  Return the group identifier to use for the process.
  */
 const char *Server::getGid ()
 {
@@ -671,7 +679,7 @@ const char *Server::getGid ()
 }
 
 /*!
- * Returns the numbers of active connections the list.
+  Returns the numbers of active connections the list.
  */
 u_long Server::getNumConnections ()
 {
@@ -679,7 +687,7 @@ u_long Server::getNumConnections ()
 }
 
 /*!
- * Returns the numbers of all the connections to the server.
+  Returns the numbers of all the connections to the server.
  */
 u_long Server::getNumTotalConnections ()
 {
@@ -687,7 +695,7 @@ u_long Server::getNumTotalConnections ()
 }
 
 /*!
- * Return a home directory object.
+  Return a home directory object.
  */
 HomeDir* Server::getHomeDir ()
 {
@@ -695,7 +703,7 @@ HomeDir* Server::getHomeDir ()
 }
 
 /*!
- * Stop the execution of the server.
+  Stop the execution of the server.
  */
 void Server::stop ()
 {
@@ -703,8 +711,8 @@ void Server::stop ()
 }
 
 /*!
- * Unload the server.
- * Return nonzero on errors.
+  Unload the server.
+  Return nonzero on errors.
  */
 int Server::terminate ()
 {
@@ -748,7 +756,7 @@ int Server::terminate ()
 
 #ifdef WIN32
   /*
-   * Under WIN32 cleanup environment strings.
+    Under WIN32 cleanup environment strings.
    */
   FreeEnvironmentStrings ((LPTSTR)envString);
 #endif
@@ -764,7 +772,7 @@ int Server::terminate ()
   clearMulticastRegistry ();
 
   /*
-   * Free all the threads.
+    Free all the threads.
    */
   threadsMutex->lock ();
   threads.clear ();
@@ -781,8 +789,8 @@ int Server::terminate ()
 }
 
 /*!
- * Get a pointer to the configuration file.  It is
- * valid only at startup!
+  Get a pointer to the configuration file.  It is
+  valid only at startup!
  */
 MainConfiguration *Server::getConfiguration ()
 {
@@ -790,9 +798,9 @@ MainConfiguration *Server::getConfiguration ()
 }
 
 /*!
- * Here is loaded the configuration of the server.
- * The configuration file is a XML file.
- * Return nonzero on errors.
+  Here is loaded the configuration of the server.
+  The configuration file is a XML file.
+  Return nonzero on errors.
  */
 int Server::initialize ()
 {
@@ -844,9 +852,7 @@ int Server::initialize ()
 
   configurationFileManager->readData (&hashedDataTrees, &hashedData);
 
-  /*
-   * Process console colors information.
-   */
+  /* Process console colors information.  */
   list<string> levels = logManager->getLoggingLevelsByNames ();
   for (list<string>::iterator it = levels.begin (); it != levels.end (); it++)
     {
@@ -866,13 +872,13 @@ int Server::initialize ()
 
   initLogManager ();
 
-  data = getData ("server.buffer_size");
+  data = getData ("server.buffer_size", "102400");
   if (data)
     buffersize = (atol (data) > 81920) ?  atol (data) :  81920 ;
 
-  data = getData ("server.connection_timeout");
+  data = getData ("connection.timeout");
   if (data)
-    connectionTimeout = MYSERVER_SEC ((u_long)atol (data));
+    connectionTimeout = MYSERVER_SEC ((u_long) atol (data));
 
   data = getData ("server.static_threads");
   if (data)
@@ -899,16 +905,16 @@ int Server::initialize ()
   /* Get the default throttling rate to use on connections.  */
   data = getData ("connection.throttling");
   if (data)
-    throttlingRate = (u_long)atoi (data);
+    throttlingRate = (u_long) atoi (data);
 
   data = getData ("server.max_log_size");
   if (data)
-    maxLogFileSize=(u_long)atol (data);
+    maxLogFileSize=(u_long) atol (data);
 
   data = getData ("server.max_files_cache");
   if (data)
     {
-      u_long maxSize = (u_long)atol (data);
+      u_long maxSize = (u_long) atol (data);
       cachedFiles.initialize (maxSize);
     }
   else
@@ -927,14 +933,14 @@ int Server::initialize ()
   data = getData ("server.max_file_cache");
   if (data)
     {
-      u_long maxSize = (u_long)atol (data);
+      u_long maxSize = (u_long) atol (data);
       cachedFiles.setMaxSize (maxSize);
     }
 
   data = getData ("server.min_file_cache");
   if (data)
     {
-      u_long minSize = (u_long)atol (data);
+      u_long minSize = (u_long) atol (data);
       cachedFiles.setMinSize (minSize);
     }
 
@@ -961,16 +967,16 @@ int Server::initialize ()
 }
 
 /*!
- * Check if there are free threads to handle a new request.  If there
- * are not enough threads create a new one.
+  Check if there are free threads to handle a new request.  If there
+  are not enough threads create a new one.
  */
 void Server::checkThreadsNumber ()
 {
   threadsMutex->lock ();
 
   /*
-   *Create a new thread if there are not available threads and
-   *we did not reach the limit.
+    Create a new thread if there are not available threads and
+    we did not reach the limit.
    */
   if ((threads.size () < nMaxThreads) && (freeThreads < 1))
     addThread (false);
@@ -979,7 +985,7 @@ void Server::checkThreadsNumber ()
 }
 
 /*!
- * Get the default throttling rate to use with connections to the server.
+  Get the default throttling rate to use with connections to the server.
  */
 u_long Server::getThrottlingRate ()
 {
@@ -987,8 +993,8 @@ u_long Server::getThrottlingRate ()
 }
 
 /*!
- * This function returns the max size of the logs file as defined in the
- * configuration file.
+  This function returns the max size of the logs file as defined in the
+  configuration file.
  */
 int Server::getMaxLogFileSize ()
 {
@@ -996,7 +1002,7 @@ int Server::getMaxLogFileSize ()
 }
 
 /*!
- * Returns the connection timeout.
+  Returns the connection timeout.
  */
 u_long Server::getTimeout ()
 {
@@ -1004,10 +1010,10 @@ u_long Server::getTimeout ()
 }
 
 /*!
- * This function add a new connection to the list.
- * On a failure, it is responsibility of the caller to
- * free S.
- * \return 0 on success.
+  This function add a new connection to the list.
+  On a failure, it is responsibility of the caller to
+  free S.
+  \return 0 on success.
  */
 int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
 {
@@ -1019,13 +1025,13 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
   int dim;
 
   /*
-   * We can use MAX_IP_STRING_LEN only because we use NI_NUMERICHOST
-   * in getnameinfo call; Otherwise we should have used NI_MAXHOST.
-   * ip is the string containing the address of the remote host connecting
-   * to the server.
-   * localIp is the local address used by the connection.
-   * port is the remote port used by the client to open the connection.
-   * myPort is the port used by the server to listen.
+    We can use MAX_IP_STRING_LEN only because we use NI_NUMERICHOST
+    in getnameinfo call; Otherwise we should have used NI_MAXHOST.
+    ip is the string containing the address of the remote host connecting
+    to the server.
+    localIp is the local address used by the connection.
+    port is the remote port used by the client to open the connection.
+    myPort is the port used by the server to listen.
    */
   if ( asockIn == NULL ||
        (asockIn->ss_family != AF_INET && asockIn->ss_family != AF_INET6))
@@ -1038,14 +1044,15 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
     return -1;
 
   /*
-   * Do not accept this connection if a MAX_CONNECTIONS_TO_ACCEPT limit is
-   * defined.
+    Do not accept this connection if a MAX_CONNECTIONS_TO_ACCEPT limit is
+    defined.
    */
   if (maxConnectionsToAccept && connectionsScheduler.getNumAliveConnections ()
       >= maxConnectionsToAccept)
     return -1;
 
 #if ( HAVE_IPV6 )
+  int ret;
   if ( asockIn->ss_family == AF_INET )
     ret = getnameinfo (reinterpret_cast<const sockaddr *>(asockIn),
                        sizeof (sockaddr_in),
@@ -1067,20 +1074,20 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
     ret = getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
                        sizeof (sockaddr_in), localIp, MAX_IP_STRING_LEN,
                        NULL, 0, NI_NUMERICHOST);
-  else// AF_INET6
+  else
     ret = getnameinfo (reinterpret_cast<const sockaddr *>(&localSockIn),
                        sizeof (sockaddr_in6), localIp, MAX_IP_STRING_LEN,
                        NULL, 0, NI_NUMERICHOST);
   if (ret)
     return -1;
-#else// !HAVE_IPV6
+#else
   dim = sizeof (localSockIn);
   s->getsockname ((MYSERVER_SOCKADDR*)&localSockIn, &dim);
   strncpy (ip,  inet_ntoa (((sockaddr_in *)asockIn)->sin_addr),
            MAX_IP_STRING_LEN);
   strncpy (localIp, inet_ntoa (((sockaddr_in *)&localSockIn)->sin_addr),
            MAX_IP_STRING_LEN);
-#endif//HAVE_IPV6
+#endif
 
   /* Port used by the client.  */
   if ( asockIn->ss_family == AF_INET )
@@ -1102,7 +1109,7 @@ int Server::addConnection (Socket *s, MYSERVER_SOCKADDRIN *asockIn)
 }
 
 /*!
- * Return the max number of threads that the server can start.
+  Return the max number of threads that the server can start.
  */
 int Server::getMaxThreads ()
 {
@@ -1110,8 +1117,8 @@ int Server::getMaxThreads ()
 }
 
 /*!
- * Add a new connection.
- *A connection is defined using a connection struct.
+  Add a new connection.
+  A connection is defined using a connection struct.
  */
 ConnectionPtr Server::addConnectionToList (Socket* s,
                                            MYSERVER_SOCKADDRIN* /*asockIn*/,
@@ -1127,8 +1134,16 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
   vector<Multicast<string, void*, int>*>* handlers;
 
   connectionsPoolLock.lock ();
-  newConnection = connectionsPool.forcedGet ();
-  connectionsPoolLock.unlock ();
+  try
+    {
+      newConnection = connectionsPool.forcedGet ();
+      connectionsPoolLock.unlock ();
+    }
+  catch (...)
+    {
+      connectionsPoolLock.unlock ();
+      throw;
+    }
 
   if (!newConnection)
     return NULL;
@@ -1143,9 +1158,17 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
   newConnection->host = vhostManager.getVHost (0, localIpAddr, localPort);
   if (newConnection->host == NULL)
     {
-      connectionsPoolLock.lock ();
-      connectionsPool.put (newConnection);
-      connectionsPoolLock.unlock ();
+      try
+        {
+          connectionsPoolLock.lock ();
+          connectionsPool.put (newConnection);
+          connectionsPoolLock.unlock ();
+        }
+      catch (...)
+        {
+          connectionsPoolLock.unlock ();
+          throw;
+        }
       return 0;
     }
 
@@ -1167,8 +1190,17 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
         if ((*handlers)[i]->updateMulticast (this, msg, newConnection) == 1)
           {
             connectionsPoolLock.lock ();
-            connectionsPool.put (newConnection);
-            connectionsPoolLock.unlock ();
+            try
+              {
+                connectionsPool.put (newConnection);
+                connectionsPoolLock.unlock ();
+              }
+            catch (...)
+              {
+                connectionsPoolLock.unlock ();
+                throw;
+              }
+
             return 0;
           }
     }
@@ -1186,8 +1218,17 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
       if (ret < 0)
         {
           connectionsPoolLock.lock ();
-          connectionsPool.put (newConnection);
-          connectionsPoolLock.unlock ();
+          try
+            {
+              connectionsPool.put (newConnection);
+              connectionsPoolLock.unlock ();
+            }
+          catch (...)
+            {
+              connectionsPoolLock.unlock ();
+              throw;
+            }
+
           delete sslSocket;
           return 0;
         }
@@ -1207,9 +1248,9 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
     connectionsScheduler.addNewWaitingConnection (newConnection);
 
   /*
-   * If defined maxConnections and the number of active connections
-   * is bigger than it say to the protocol that will parse the connection
-   * to remove it from the active connections list.
+    If defined maxConnections and the number of active connections
+    is bigger than it say to the protocol that will parse the connection
+    to remove it from the active connections list.
    */
   if (maxConnections
       && connectionsScheduler.getNumAliveConnections () > maxConnections)
@@ -1221,7 +1262,7 @@ ConnectionPtr Server::addConnectionToList (Socket* s,
 }
 
 /*!
- * Delete a connection.
+  Delete a connection.
  */
 int Server::deleteConnection (ConnectionPtr s)
 {
@@ -1231,14 +1272,22 @@ int Server::deleteConnection (ConnectionPtr s)
   s->destroy ();
 
   connectionsPoolLock.lock ();
-  connectionsPool.put (s);
-  connectionsPoolLock.unlock ();
+  try
+    {
+      connectionsPool.put (s);
+      connectionsPoolLock.unlock ();
+    }
+  catch (...)
+    {
+      connectionsPoolLock.unlock ();
+      throw;
+    }
 
   return 0;
 }
 
 /*!
- * Get notified when a connection is closed.
+  Get notified when a connection is closed.
  */
 int Server::notifyDeleteConnection (ConnectionPtr s)
 {
@@ -1255,7 +1304,7 @@ int Server::notifyDeleteConnection (ConnectionPtr s)
 }
 
 /*!
- * Get a connection to parse.
+  Get a connection to parse.
  */
 ConnectionPtr Server::getConnection (int /*id*/)
 {
@@ -1263,7 +1312,7 @@ ConnectionPtr Server::getConnection (int /*id*/)
 }
 
 /*!
- * Delete all the active connections.
+  Delete all the active connections.
  */
 void Server::clearAllConnections ()
 {
@@ -1285,8 +1334,8 @@ void Server::clearAllConnections ()
 
 
 /*!
- * Returns the full path of the binaries directory.
- * The directory where the file myserver (.exe) is.
+  Returns the full path of the binaries directory.
+  The directory where the file myserver (.exe) is.
  */
 const char *Server::getPath ()
 {
@@ -1294,7 +1343,7 @@ const char *Server::getPath ()
 }
 
 /*!
- * Add a free thread.
+  Add a free thread.
  */
 void Server::increaseFreeThread ()
 {
@@ -1304,7 +1353,7 @@ void Server::increaseFreeThread ()
 }
 
 /*!
- * Remove a free thread.
+  Remove a free thread.
  */
 void Server::decreaseFreeThread ()
 {
@@ -1315,7 +1364,7 @@ void Server::decreaseFreeThread ()
 
 
 /*!
- * Returns the name of the server (the name of the current PC).
+  Returns the name of the server (the name of the current PC).
  */
 const char *Server::getServerName ()
 {
@@ -1323,20 +1372,28 @@ const char *Server::getServerName ()
 }
 
 /*!
- * Gets the number of threads.
- */
-u_long Server::getNumThreads ()
+  Get Information on the number of threads.
+  \param num if not NULL will contain the number of active threads.
+  \param max if not NULL will contain the maximum number of threads.
+  \param staticThreads if not NULL will contain the maximum number of always
+  active threads.
+*/
+void Server::getThreadsNumberInformation (u_long *num, u_long *max,
+                                          u_long *staticThreads)
 {
-  u_long ret;
-  threadsMutex->lock ();
-  ret = threads.size ();
-  threadsMutex->unlock ();
-  return ret;
+  if (num)
+    *num = threads.size ();
+
+  if (max)
+    *max = nMaxThreads;
+
+  if (staticThreads)
+    *staticThreads = nStaticThreads;
 }
 
 /*!
- * Returns a comma-separated local machine IPs list.
- * For example: 192.168.0.1, 61.62.63.64, 65.66.67.68.69
+  Returns a comma-separated local machine IPs list.
+  For example: 192.168.0.1, 61.62.63.64, 65.66.67.68.69
  */
 const char *Server::getAddresses ()
 {
@@ -1344,8 +1401,8 @@ const char *Server::getAddresses ()
 }
 
 /*!
- * Clear the data dictionary.
- * Returns zero on success.
+  Clear the data dictionary.
+  Returns zero on success.
  */
 int Server::freeHashedData ()
 {
@@ -1369,8 +1426,8 @@ int Server::freeHashedData ()
 }
 
 /*!
- * Get the value for name in the hash dictionary.
- * If the value is not found then defValue is returned.
+  Get the value for name in the hash dictionary.
+  If the value is not found then defValue is returned.
  */
 const char *Server::getData (const char *name, const char *defValue)
 {
@@ -1380,7 +1437,7 @@ const char *Server::getData (const char *name, const char *defValue)
 }
 
 /*!
- * Get the specified protocol.
+  Get the specified protocol.
  */
 Protocol* Server::getProtocol (const char *protocolName)
 {
@@ -1390,13 +1447,13 @@ Protocol* Server::getProtocol (const char *protocolName)
 }
 
 /*!
- * If specified set the uid/gid for the process.
+  If specified set the uid/gid for the process.
  */
 void Server::setProcessPermissions ()
 {
   /*
-   * If the configuration specify a group id, change the current group for
-   * the process.
+    If the configuration specify a group id, change the current group for
+    the process.
    */
   if (gid.length ())
     {
@@ -1412,9 +1469,9 @@ void Server::setProcessPermissions ()
     }
 
   /*
-   * If the configuration file provides a user identifier, change the
-   * current user for the process. Disable the reboot when this feature
-   * is used.
+    If the configuration file provides a user identifier, change the
+    current user for the process. Disable the reboot when this feature
+    is used.
    */
   if (uid.length ())
     {
@@ -1427,8 +1484,8 @@ void Server::setProcessPermissions ()
 }
 
 /*!
- * Reboot the server.
- * Returns non zero on errors.
+  Reboot the server.
+  Returns non zero on errors.
  */
 int Server::reboot ()
 {
@@ -1494,7 +1551,7 @@ int Server::reboot ()
 }
 
 /*!
- * Returns if the server is ready.
+  Returns if the server is ready.
  */
 bool Server::isServerReady ()
 {
@@ -1502,7 +1559,7 @@ bool Server::isServerReady ()
 }
 
 /*!
- * Reboot the server on the next loop.
+  Reboot the server on the next loop.
  */
 void Server::delayedReboot ()
 {
@@ -1511,7 +1568,7 @@ void Server::delayedReboot ()
 }
 
 /*!
- * Return the factory object to create cached files.
+  Return the factory object to create cached files.
  */
 CachedFileFactory* Server::getCachedFiles ()
 {
@@ -1519,7 +1576,7 @@ CachedFileFactory* Server::getCachedFiles ()
 }
 
 /*!
- * Get a list with all the alive connections.
+  Get a list with all the alive connections.
  */
 void Server::getConnections (list<ConnectionPtr>& out)
 {
@@ -1527,7 +1584,7 @@ void Server::getConnections (list<ConnectionPtr>& out)
 }
 
 /*!
- * Disable the autoreboot.
+  Disable the autoreboot.
  */
 void Server::disableAutoReboot ()
 {
@@ -1535,7 +1592,7 @@ void Server::disableAutoReboot ()
 }
 
 /*!
- * Enable the autoreboot
+  Enable the autoreboot
  */
 void Server::enableAutoReboot ()
 {
@@ -1543,7 +1600,7 @@ void Server::enableAutoReboot ()
 }
 
 /*!
- * Return the ProtocolManager object.
+  Return the ProtocolManager object.
  */
 ProtocolsManager *Server::getProtocolsManager ()
 {
@@ -1551,7 +1608,7 @@ ProtocolsManager *Server::getProtocolsManager ()
 }
 
 /*!
- * Return nonzero if the autoreboot is enabled.
+  Return nonzero if the autoreboot is enabled.
  */
 bool Server::isAutorebootEnabled ()
 {
@@ -1559,7 +1616,7 @@ bool Server::isAutorebootEnabled ()
 }
 
 /*!
- * Create a new thread.
+  Create a new thread.
  */
 int Server::addThread (bool staticThread)
 {
@@ -1596,7 +1653,6 @@ int Server::addThread (bool staticThread)
   newThread = new ClientsThread (this);
 
   handlers = getHandlers (msg);
-
   if (handlers)
     {
       for (size_t i = 0; i < handlers->size (); i++)
@@ -1607,7 +1663,7 @@ int Server::addThread (bool staticThread)
 
   newThread->setStatic (staticThread);
 
-  newThread->id = (u_long)(++currentThreadID);
+  newThread->id = (u_long) (++currentThreadID);
 
   ret = newThread->run ();
 
@@ -1622,8 +1678,8 @@ int Server::addThread (bool staticThread)
 }
 
 /*!
- * Remove a thread.
- * Return zero if a thread was removed.
+  Remove a thread.
+  Return zero if a thread was removed.
  */
 int Server::removeThread (u_long ID)
 {
@@ -1660,9 +1716,9 @@ int Server::removeThread (u_long ID)
 }
 
 /*!
- * Create the class instance.  Call this before use
- * the Server class.  The instance is not created in
- * getInstance to have a faster inline function.
+  Create the class instance.  Call this before use
+  the Server class.  The instance is not created in
+  getInstance to have a faster inline function.
  */
 void Server::createInstance ()
 {
@@ -1671,7 +1727,7 @@ void Server::createInstance ()
 }
 
 /*!
- * Get a pointer to a filters factory object.
+  Get a pointer to a filters factory object.
  */
 FiltersFactory* Server::getFiltersFactory ()
 {
@@ -1679,7 +1735,7 @@ FiltersFactory* Server::getFiltersFactory ()
 }
 
 /*!
- * Check how many threads are not working.
+  Check how many threads are not working.
  */
 int Server::countAvailableThreads ()
 {
@@ -1695,10 +1751,10 @@ int Server::countAvailableThreads ()
 }
 
 /*!
- * Write a message on a single LogStream specifying a formatting string.
- *
- * \see LogManager#log (void*, string, string, LoggingLeve, bool, va_list)
- * \return 0 on success, 1 on error.
+  Write a message on a single LogStream specifying a formatting string.
+
+  \see LogManager#log (void*, string, string, LoggingLeve, bool, va_list)
+  \return 0 on success, 1 on error.
  */
 int Server::log (LoggingLevel level, const char *fmt, ...)
 {
@@ -1719,7 +1775,7 @@ int Server::log (LoggingLevel level, const char *fmt, ...)
 }
 
 /*!
- * Write a string to the log file and terminate the line.
+  Write a string to the log file and terminate the line.
  */
 int Server::log (char const* str, LoggingLevel level)
 {
@@ -1727,7 +1783,7 @@ int Server::log (char const* str, LoggingLevel level)
 }
 
 /*!
- * Use a specified location as log.
+  Use a specified location as log.
  */
 int
 Server::setLogLocation (string location)
@@ -1743,7 +1799,7 @@ Server::setLogLocation (string location)
 }
 
 /*!
- * Get the size for the first buffer.
+  Get the size for the first buffer.
  */
 u_long Server::getBuffersize ()
 {
