@@ -1,21 +1,21 @@
 /*
- MyServer
- Copyright (C) 2008, 2009 Free Software Foundation, Inc.
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 3 of the License, or
- (at your option) any later version.
+  MyServer
+  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
-#include "stdafx.h"
+#include "myserver.h"
 #include <ctype.h>
 
 #include <cppunit/CompilerOutputter.h>
@@ -86,21 +86,21 @@ public:
 
   void testDirectory ()
   {
-    string dir ("foo");
+    const string dir ("foo");
     securityToken->setDirectory (&dir);
     CPPUNIT_ASSERT_EQUAL (securityToken->getDirectory (), &dir);
   }
 
   void testSysDirectory ()
   {
-    string sysDir ("foo");
+    const string sysDir ("foo");
     securityToken->setSysDirectory (&sysDir);
     CPPUNIT_ASSERT_EQUAL (securityToken->getSysDirectory (), &sysDir);
   }
 
   void testResource ()
   {
-    string resource ("foo");
+    const string resource ("foo");
     securityToken->setResource (&resource);
     CPPUNIT_ASSERT_EQUAL (securityToken->getResource (), &resource);
   }
