@@ -1,19 +1,21 @@
 /*
- MyServer
- Copyright (C) 2008, 2009 Free Software Foundation, Inc.
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 3 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful, 
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+  MyServer
+  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+#include "myserver.h"
 
 #define TESTSERVERPEM "server.pem"
 #define TESTSERVERKEY "server.key"
@@ -23,20 +25,21 @@
 #include <cppunit/ui/text/TestRunner.h>
 #include <cppunit/extensions/HelperMacros.h>
 
-#include "../include/base/socket/ssl_socket.h"
-#include "../include/base/file/file.h"
-#include "../include/base/file/files_utility.h"
-#include "../include/base/thread/thread.h"
+#include <include/base/socket/socket.h>
+#include <include/base/socket/ssl_socket.h>
+#include <include/base/file/file.h>
+#include <include/base/file/files_utility.h>
+#include <include/base/thread/thread.h>
+#include <include/base/utility.h>
 
-extern "C"
-{
 #include <string.h>
 #ifndef WIN32
-#include <errno.h>
-#include <arpa/inet.h>
+# include <errno.h>
+# include <arpa/inet.h>
 #endif
-}
 
+
+#include <string>
 #include <iostream>
 using namespace std;
 
@@ -87,161 +90,183 @@ FX3J22wtVUi4Ve/XftYt6RJKd764o5WTdh/Z+RUbtusXnj3ygpI/G7fTzuPUj9uF\n\
 3K2VTrZpJnbMs7+i3w/ziC/cqWRVK6Rcq3bLzTXrig==\n\
 -----END CERTIFICATE-----\n";
 
-static DEFINE_THREAD(testSslRecvClient, pParam);
+static DEFINE_THREAD (testSslRecvClient, pParam);
+
+struct SslThreadArg
+{
+  int port;
+  bool success;
+
+  /* This is valid only if !success.  */
+  exception reason;
+};
 
 class TestSslSocket : public CppUnit::TestFixture
 {
-  CPPUNIT_TEST_SUITE( TestSslSocket );
-  
-  CPPUNIT_TEST( testRecv );
-  
-  CPPUNIT_TEST_SUITE_END( );
-  
+  CPPUNIT_TEST_SUITE (TestSslSocket);
+  CPPUNIT_TEST (testRecv);
+  CPPUNIT_TEST_SUITE_END ();
+
 public:
-  
-  void setUp ( )
+
+  void setUp ()
   {
     initializeSSL ();
-    u_long nbw;
+    size_t nbw;
     File f;
 
-    f.openFile (TESTSERVERKEY,  File::WRITE | File::FILE_CREATE_ALWAYS);
+    f.openFile (TESTSERVERKEY,  File::WRITE | File::FILE_OPEN_ALWAYS);
     f.writeToFile (serverKey, strlen (serverKey), &nbw);
     f.close ();
 
-    f.openFile (TESTSERVERPEM,  File::WRITE | File::FILE_CREATE_ALWAYS);
+    f.openFile (TESTSERVERPEM,  File::WRITE | File::FILE_OPEN_ALWAYS);
     f.writeToFile (serverPem, strlen (serverPem), &nbw);
     f.close ();
   }
-  
-  void tearDown ( )
+
+  void tearDown ()
   {
     FilesUtility::deleteFile (TESTSERVERPEM);
     FilesUtility::deleteFile (TESTSERVERKEY);
   }
-  
-  void testRecv ( )
+
+  void testRecv ()
   {
     Socket *obj = new Socket;
     SslSocket *sslObj = NULL;
     SSL_CTX *ctx = NULL;;
     ThreadID tid;
-    
+
     int optvalReuseAddr = 1;
-    char host[] = "localhost";
     int port = 6543;
     MYSERVER_SOCKADDRIN sockIn = { 0 };
     int status;
-    
+
     ((sockaddr_in*) (&sockIn))->sin_family = AF_INET;
-    ((sockaddr_in*) (&sockIn))->sin_addr.s_addr = inet_addr ( "127.0.0.1" );
-    ((sockaddr_in*) (&sockIn))->sin_port = htons ( port );
-    
-    socklen_t sockInLen = sizeof ( sockaddr_in );
-    
-    CPPUNIT_ASSERT( obj->socket ( AF_INET, SOCK_STREAM, 0 ) != -1 );
-    
-    CPPUNIT_ASSERT( obj->setsockopt ( SOL_SOCKET, SO_REUSEADDR,
+    ((sockaddr_in*) (&sockIn))->sin_addr.s_addr = inet_addr (LOCALHOST_ADDRESS);
+    ((sockaddr_in*) (&sockIn))->sin_port = htons (port);
+
+    socklen_t sockInLen = sizeof (sockaddr_in);
+
+    CPPUNIT_ASSERT (obj->socket (AF_INET, SOCK_STREAM, 0) != -1);
+
+    CPPUNIT_ASSERT (obj->setsockopt (SOL_SOCKET, SO_REUSEADDR,
                                       (const char*) &optvalReuseAddr,
-                                      sizeof(optvalReuseAddr) ) != -1 );
-    
-    ctx = SSL_CTX_new ( SSLv23_server_method() );
+                                      sizeof (optvalReuseAddr)) != -1);
 
-    if ( SSL_CTX_use_certificate_file ( ctx, TESTSERVERPEM, SSL_FILETYPE_PEM ) != 1 )
+    ctx = SSL_CTX_new (SSLv23_server_method ());
+
+    if (SSL_CTX_use_certificate_file (ctx, TESTSERVERPEM, SSL_FILETYPE_PEM) != 1)
     {
-      SSL_CTX_free ( ctx );
+      SSL_CTX_free (ctx);
       CPPUNIT_ASSERT (false);
     }
 
-    if ( SSL_CTX_use_PrivateKey_file ( ctx, TESTSERVERKEY, SSL_FILETYPE_PEM ) != 1 )
+    if (SSL_CTX_use_PrivateKey_file (ctx, TESTSERVERKEY, SSL_FILETYPE_PEM) != 1)
     {
-      SSL_CTX_free ( ctx );
+      SSL_CTX_free (ctx);
       CPPUNIT_ASSERT (false);
     }
 
-    // If the port is used by another program, try a few others.
+    /* If the port is used by another program, try a few others.  */
     do
-    {
-      port++;
-      ((sockaddr_in*) (&sockIn))->sin_port = htons ( port);
-    }
+      {
+        port++;
+        ((sockaddr_in*) (&sockIn))->sin_port = htons (port);
+      }
     while ((status = obj->bind (&sockIn, sockInLen)) != 0 && port < 28000);
 
-    CPPUNIT_ASSERT( status != -1 );
+    CPPUNIT_ASSERT (status != -1);
 
-    CPPUNIT_ASSERT( obj->listen ( 1 ) != -1 );
+    CPPUNIT_ASSERT (obj->listen (1) != -1);
+
+    SslThreadArg arg;
+    arg.port = port;
+    CPPUNIT_ASSERT_EQUAL (Thread::create (&tid, testSslRecvClient, &arg), 0);
+
+    Thread::wait (MYSERVER_SEC (1));
+
+    Socket s = obj->accept (&sockIn, &sockInLen);
 
-    CPPUNIT_ASSERT_EQUAL( Thread::create ( &tid, testSslRecvClient, &port ), 0 );
-    Socket s = obj->accept ( &sockIn, &sockInLen );
     sslObj = new SslSocket (&s);
+    sslObj->setSSLContext (ctx);
 
-    sslObj->setSSLContext ( ctx );
-
-    int ret = sslObj->sslAccept ( );
-
-    if ( ret < 0 )
+    int ret = sslObj->sslAccept ();
+    if (ret < 0)
     {
       delete obj;
       delete sslObj;
-      SSL_CTX_free ( ctx );
+      SSL_CTX_free (ctx);
       CPPUNIT_ASSERT (false);
     }
-    
+
     char buf[32] = {0};
-    
-    ret = sslObj->recv ( buf, sizeof(buf), 0 );
 
-    sslObj->send ( "a", 1, 0);
+    ret = sslObj->recv (buf, sizeof (buf), 0);
 
-    CPPUNIT_ASSERT(ret != -1 );
+    sslObj->send ("a", 1, 0);
 
-    CPPUNIT_ASSERT( sslObj->close ( ) != -1 );
+    CPPUNIT_ASSERT (ret != -1);
 
-    CPPUNIT_ASSERT( obj->close ( ) != -1 );
+    CPPUNIT_ASSERT (sslObj->close () != -1);
 
-    SSL_CTX_free ( ctx );
+    CPPUNIT_ASSERT (obj->close () != -1);
+
+    SSL_CTX_free (ctx);
+
+    Thread::join (tid);
+    if (!arg.success)
+      throw arg.reason;
 
     delete obj;
     delete sslObj;
   }
 };
 
-CPPUNIT_TEST_SUITE_REGISTRATION( TestSslSocket );
+CPPUNIT_TEST_SUITE_REGISTRATION (TestSslSocket);
 
-static DEFINE_THREAD(testSslRecvClient, pParam)
+static DEFINE_THREAD (testSslRecvClient, pParam)
 {
-  Socket *obj2 = new Socket;
-  SslSocket *sslObj2 = NULL;
-  int optvalReuseAddr = 1;
-  char host[] = "localhost";
-  int port = *((int*)pParam);
-  int cstatus = 0;
+  SslSocket *sslClient = NULL;
+  int err;
+  SslThreadArg *arg = (SslThreadArg *) pParam;
   MYSERVER_SOCKADDRIN sockIn = { 0 };
 
   ((sockaddr_in*) (&sockIn))->sin_family = AF_INET;
-  ((sockaddr_in*) (&sockIn))->sin_addr.s_addr = inet_addr ( "127.0.0.1" );
-  ((sockaddr_in*) (&sockIn))->sin_port = htons ( port );
+  ((sockaddr_in*) (&sockIn))->sin_addr.s_addr = inet_addr (LOCALHOST_ADDRESS);
+  ((sockaddr_in*) (&sockIn))->sin_port = htons (arg->port);
 
-  int sockInLen = sizeof ( struct sockaddr_in );
+  int sockInLen = sizeof (struct sockaddr_in);
 
-  CPPUNIT_ASSERT( obj2->socket ( AF_INET, SOCK_STREAM, 0 ) != -1 );
+  arg->success = true;
 
-  sslObj2 = new SslSocket ( obj2 );
+  sslClient = new SslSocket ();
 
-  CPPUNIT_ASSERT( sslObj2->connect ( &sockIn, sockInLen ) != -1 );
+  try
+    {
+      err = sslClient->socket (AF_INET, SOCK_STREAM, 0);
+      CPPUNIT_ASSERT (err != -1);
 
-  char buf[] = "Works?\n";
+      err = sslClient->connect (&sockIn, sockInLen);
+      CPPUNIT_ASSERT (err != -1);
 
-  int ret = sslObj2->send ( buf, strlen(buf), 0 );
+      char buf[] = "Works?\n";
 
-  sslObj2->recv ( buf, 1, 0 );
+      err = sslClient->send (buf, strlen (buf), 0);
+      sslClient->recv (buf, 1, 0);
 
-  CPPUNIT_ASSERT(ret != -1 );
+      CPPUNIT_ASSERT (err != -1);
+      CPPUNIT_ASSERT (sslClient->close () != -1);
+      delete sslClient;
+    }
+  catch (exception& e)
+    {
+      arg->success = false;
+      arg->reason = e;
+    }
+  catch (...)
+    {
+    }
 
-
-  CPPUNIT_ASSERT( sslObj2->close ( ) != -1 );
-
-  delete obj2;
-  delete sslObj2;
   return 0;
 }
